@@ -1,11 +1,11 @@
 /*
  *   CUPS Backend common code
  *
- *   Copyright (c) 2007-2020 Solomon Peachy <pizza@shaftnet.org>
+ *   Copyright (c) 2007-2025 Solomon Peachy <pizza@shaftnet.org>
  *
  *   The latest version of this program can be found at:
  *
- *     http://git.shaftnet.org/cgit/selphy_print.git
+ *     https://git.shaftnet.org/gitea/slp/selphy_print.git
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the Free
@@ -18,9 +18,7 @@
  *   for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
- *
- *          [http://www.gnu.org/licenses/gpl-2.0.html]
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  *   SPDX-License-Identifier: GPL-2.0+
  *
@@ -31,10 +29,7 @@
 #include <signal.h>
 #include <strings.h>  /* For strncasecmp */
 
-#define BACKEND_VERSION "0.110G"
-#ifndef URI_PREFIX
-#error "Must Define URI_PREFIX"
-#endif
+#define BACKEND_VERSION "0.134G"
 
 #ifndef CORRTABLE_PATH
 #ifdef PACKAGE_DATA_DIR
@@ -62,12 +57,18 @@ int ncopies = 1;
 int collate = 0;
 int test_mode = 0;
 int quiet = 0;
+int stats_only = 0;
 FILE *logger;
 
 const char *corrtable_path = CORRTABLE_PATH;
 static int max_xfer_size = URB_XFER_SIZE;
 static int xfer_timeout = XFER_TIMEOUT;
+
+#if defined(OLD_URI) && OLD_URI
+static int old_uri = 1;
+#else
 static int old_uri = 0;
+#endif
 
 /* Support Functions */
 int backend_claim_interface(struct libusb_device_handle *dev, int iface,
@@ -205,6 +206,9 @@ int parse1284_data(const char *device_id, struct deviceid_dict* dict)
 		*ptr = 0;
 		device_id++;
 
+		if (!strlen(key) || !strlen(val))
+			continue;
+
 		/* Add it to the dictionary */
 		dict[num].key = strdup(key);
 		dict[num].val = strdup(val);
@@ -251,7 +255,7 @@ int read_data(struct dyesub_connection *conn, uint8_t *buf, int buflen, int *rea
 		DEBUG("Received %d bytes from printer\n", *readlen);
 	}
 
-	if ((dyesub_debug > 1 && buflen < 4096) ||
+	if ((dyesub_debug > 1 && *readlen < 4096) ||
 	    dyesub_debug > 2) {
 		int i = *readlen;
 
@@ -339,7 +343,7 @@ static char *sanitize_string(char *str) {
 
    These functions are Public Domain code obtained from:
 
-   http://www.geekhideout.com/urlcode.shtml
+   https://www.geekhideout.com/urlcode.shtml
 
 */
 #include <ctype.h>  /* for isalnum() */
@@ -421,7 +425,6 @@ static int probe_device(struct libusb_device *device,
 	uint8_t endp_up, endp_down;
 
 	DEBUG("Probing VID: %04X PID: %04x\n", desc->idVendor, desc->idProduct);
-	STATE("+connecting-to-device\n");
 
 	if ((i = libusb_open(device, &dev))) {
 #ifdef _WIN32
@@ -466,7 +469,9 @@ static int probe_device(struct libusb_device *device,
 				continue;
 			}
 
-#if 1
+			/* Note we actually only match on explicit VID+PIDs so there's no need to filter based
+			   on specific class/type */
+
 			/* Explicitly exclude IPP-over-USB interfaces */
 			if (desc->bDeviceClass == LIBUSB_CLASS_PER_INTERFACE &&
 			    config->interface[iface].altsetting[altset].bInterfaceClass == LIBUSB_CLASS_PRINTER &&
@@ -474,16 +479,6 @@ static int probe_device(struct libusb_device *device,
 			    config->interface[iface].altsetting[altset].bInterfaceProtocol == USB_INTERFACE_PROTOCOL_IPP) {
 				continue;
 			}
-#else
-			// Make sure it's a printer class device that supports bidir comms  (XXX Is this necessarily true?)
-			if (desc->bDeviceClass == LIBUSB_CLASS_PRINTER ||
-			    (desc->bDeviceClass == LIBUSB_CLASS_PER_INTERFACE &&
-			     config->interface[iface].altsetting[altset].bInterfaceClass == LIBUSB_CLASS_PRINTER &&
-			     config->interface[iface].altsetting[altset].bInterfaceSubClass == USB_SUBCLASS_PRINTER &&
-			     config->interface[iface].altsetting[altset].bInterfaceProtocol != USB_INTERFACE_PROTOCOL_BIDIR)) {
-				continue;
-			}
-#endif
 
 			/* Find the first set of endpoints! */
 			endp_up = endp_down = 0;
@@ -537,34 +532,44 @@ candidate:
 		dlen = parse1284_data(ieee_id, dict);
 	}
 
-//	if (!old_uri) goto bypass;
+	if (!old_uri && !scan_only && !dyesub_debug) goto skip_manuf_model;
 
-	/* Look up mfg string. */
+	/* Look up mfg string in IEEE1284 data */
 	if (manuf_override && strlen(manuf_override)) {
-		manuf = url_encode(manuf_override);  /* Backend supplied */
+		manuf = url_encode(manuf_override);
 	} else if ((manuf = dict_find("MANUFACTURER", dlen, dict))) {
 		manuf = url_encode(manuf);
 	} else if ((manuf = dict_find("MFG", dlen, dict))) {
 		manuf = url_encode(manuf);
 	} else if ((manuf = dict_find("MFR", dlen, dict))) {
 		manuf = url_encode(manuf);
-	} else if (desc->iManufacturer) { /* Get from USB descriptor */
+	}
+
+	/* If no manufacturer string, fall back to USB iManufacturer */
+	if ((!manuf || !strlen(manuf)) &&
+	    desc->iManufacturer) {
 		buf[0] = 0;
 		libusb_get_string_descriptor_ascii(dev, desc->iManufacturer, (unsigned char*)buf, STR_LEN_MAX);
 		sanitize_string(buf);
 		manuf = url_encode(buf);
 	}
+
 	if (!manuf || !strlen(manuf)) {  /* Last-ditch */
 		if (manuf) free(manuf);
+		WARNING("**** THIS PRINTER DOES NOT REPORT A VALID MANUFACTURER STRING!\n");
 		manuf = url_encode("Unknown"); // XXX use USB VID?
 	}
 
-	/* Look up model number */
+	/* Look up model string in IEEE1284 data */
 	if ((product = dict_find("MODEL", dlen, dict))) {
 		product = url_encode(product);
 	} else if ((product = dict_find("MDL", dlen, dict))) {
 		product = url_encode(product);
-	} else if (desc->iProduct) {  /* Get from USB descriptor */
+	}
+
+	/* If no manufacturer string, fall back to USB iProduct */
+	if ((!product || !strlen(product)) &&
+	    desc->iProduct) {
 		buf[0] = 0;
 		libusb_get_string_descriptor_ascii(dev, desc->iProduct, (unsigned char*)buf, STR_LEN_MAX);
 		sanitize_string(buf);
@@ -573,15 +578,17 @@ candidate:
 
 	if (!product || !strlen(product)) { /* Last-ditch */
 		if (!product) free(product);
+		WARNING("**** THIS PRINTER DOES NOT REPORT A VALID MODEL STRING!\n");
 		product = url_encode("Unknown"); // XXX Use USB PID?
 	}
 
-	/* Look up description */
+	/* Look up decription string in IEEE1284 data */
 	if ((descr = dict_find("DESCRIPTION", dlen, dict))) {
 		descr = strdup(descr);
 	} else if ((descr = dict_find("DES", dlen, dict))) {
 		descr = strdup(descr);
 	}
+
 	if (!descr || !strlen(descr)) { /* Last-ditch, generate */
 		char *product2 = url_decode(product);
 		char *manuf3 = url_decode(manuf);
@@ -600,9 +607,9 @@ candidate:
 		free(manuf3);
 	}
 
-//bypass:
+skip_manuf_model:
 
-	/* Look up serial number */
+	/* Prefer IEEE1284-reported serial number */
 	if ((serial = dict_find("SERIALNUMBER", dlen, dict))) {
 		serial = url_encode(serial);
 	} else if ((serial = dict_find("SN", dlen, dict))) {
@@ -611,23 +618,37 @@ candidate:
 		serial = url_encode(serial);
 	} else if ((serial = dict_find("SERN", dlen, dict))) {
 		serial = url_encode(serial);
-	} else if (!(backend->flags & BACKEND_FLAG_BADISERIAL) &&
-		   desc->iSerialNumber) {  /* Get from USB descriptor, if we can trust it.. */
+	}
+
+	/* If it's not valid, fall back to USB iSerial */
+	if ((!serial || !strlen(serial)) &&
+	    !(backend->flags & BACKEND_FLAG_BADISERIAL) &&
+	    desc->iSerialNumber) {
 		libusb_get_string_descriptor_ascii(dev, desc->iSerialNumber, (unsigned char*)buf, STR_LEN_MAX);
 		sanitize_string(buf);
 		serial = url_encode(buf);
-	} else if (backend->query_serno) { /* Get from backend hook */
+	}
+
+	/* TODO: What about situations where iSerial does not match IEEE1284?
+	         Or if the '1284 data is bogus? */
+
+	/* If still no serial, fall back to backend hook */
+	if ((!serial || !strlen(serial)) &&
+	    backend->query_serno) { /* Get from backend hook */
 		struct dyesub_connection c2;
 		c2.dev = dev;
 		c2.iface = iface;
 		c2.altset = altset;
 		c2.endp_up = endp_up;
 		c2.endp_down = endp_down;
+		c2.type = lookup_printer_type(backend,
+						desc->idVendor, desc->idProduct);
 		backend->query_serno(&c2, buf, STR_LEN_MAX);
 		serial = url_encode(buf);
 	}
 
-	if (!serial || !strlen(serial)) {  /* Last-ditch */
+	/* Last-ditch serial number fallback */
+	if (!serial || !strlen(serial)) {
 		if (serial) free(serial);
 		WARNING("**** THIS PRINTER DOES NOT REPORT A SERIAL NUMBER!\n");
 		WARNING("**** If you intend to use multiple printers of this type, you\n");
@@ -664,15 +685,20 @@ candidate:
 		found = -1;
 	}
 
+	uint8_t bus_num = libusb_get_bus_number(device);
+	uint8_t port_num = libusb_get_port_number(device);
+
 	if (dyesub_debug)
-		DEBUG("VID: %04X PID: %04X Manuf: '%s' Product: '%s' Serial: '%s' found: %d\n",
-		      desc->idVendor, desc->idProduct, manuf, product, serial, found);
+		DEBUG("VID/PID %04X/%04X @ bus/port %03d/%03d Manuf: '%s' Product: '%s' Serial: '%s' found: %d\n",
+		      desc->idVendor, desc->idProduct, bus_num, port_num, manuf, product, serial, found);
 
 	if (found != -1 && conn) {
 		conn->iface = iface;
 		conn->altset = altset;
 		conn->endp_up = endp_up;
 		conn->endp_down = endp_down;
+		conn->bus_num = bus_num;
+		conn->port_num = port_num;
 	}
 
 	/* Free things up */
@@ -699,8 +725,6 @@ abort:
 		free (dict[dlen].val);
 	}
 
-	STATE("-connecting-to-device\n");
-
 	return found;
 }
 
@@ -717,6 +741,7 @@ extern struct dyesub_backend sonyupdneo_backend;
 extern struct dyesub_backend kodak6800_backend;
 extern struct dyesub_backend kodak605_backend;
 extern struct dyesub_backend kodak1400_backend;
+extern struct dyesub_backend kodak8800_backend;
 extern struct dyesub_backend shinkos1245_backend;
 extern struct dyesub_backend shinkos2145_backend;
 extern struct dyesub_backend shinkos6145_backend;
@@ -737,6 +762,7 @@ static struct dyesub_backend *backends[] = {
 	&kodak6800_backend,
 	&kodak605_backend,
 	&kodak1400_backend,
+	&kodak8800_backend,
 	&shinkos1245_backend,
 	&shinkos2145_backend,
 	&shinkos6145_backend,
@@ -753,7 +779,7 @@ static struct dyesub_backend *backends[] = {
 	NULL,
 };
 
-static int find_and_enumerate(struct libusb_context *ctx,
+static int find_and_enumerate(const char *argv0,
 			      struct libusb_device ***list,
 			      const struct dyesub_backend *backend,
 			      const char *match_serno,
@@ -777,7 +803,7 @@ static int find_and_enumerate(struct libusb_context *ctx,
 	STATE("+org.gutenprint.searching-for-device\n");
 
 	/* Enumerate and find suitable device */
-	num = libusb_get_device_list(ctx, list);
+	num = libusb_get_device_list(NULL, list);
 
 	/* See if we can actually match on the supplied make! */
 	if (backend && make) {
@@ -836,7 +862,7 @@ static int find_and_enumerate(struct libusb_context *ctx,
 
 	match:
 		found = probe_device((*list)[i], &desc, (foundmake ? foundmake : make),
-				     URI_PREFIX, backends[k]->devices[j].manuf_str,
+				     argv0, backends[k]->devices[j].manuf_str,
 				     found, num_claim_attempts,
 				     scan_only, match_serno,
 				     conn,
@@ -885,7 +911,7 @@ static int query_markers(const struct dyesub_backend *backend, void *ctx, int fu
 	if (!backend->query_markers)
 		return CUPS_BACKEND_OK;
 
-	if (test_mode >= TEST_MODE_NOPRINT)
+	if (test_mode >= TEST_MODE_NOATTACH)
 		return CUPS_BACKEND_OK;
 
 	ret = backend->query_markers(ctx, &markers, &marker_count);
@@ -990,7 +1016,7 @@ static void dump_stats(struct dyesub_backend *backend, struct printerstats *stat
 void print_license_blurb(void)
 {
 	const char *license = "\n\
-Copyright 2007-2020 Solomon Peachy <pizza AT shaftnet DOT org>\n\
+Copyright 2007-2025 Solomon Peachy <pizza AT shaftnet DOT org>\n\
 \n\
 This program is free software; you can redistribute it and/or modify it\n\
 under the terms of the GNU General Public License as published by the Free\n\
@@ -1010,15 +1036,12 @@ along with this program; if not, see <https://www.gnu.org/licenses/>.\n\n";
 
 void print_help(const char *argv0, const struct dyesub_backend *backend)
 {
-	struct libusb_context *ctx = NULL;
 	struct libusb_device **list = NULL;
 
 	const char *ptr = getenv("BACKEND");
 	if (!ptr)
-		ptr = strrchr(argv0, '/');
-	if (ptr)
-		ptr++;
-	else
+		ptr = getenv("DYESUB_BACKEND");
+	if (!ptr)
 		ptr = argv0;
 
 	if (!backend)
@@ -1029,10 +1052,10 @@ void print_help(const char *argv0, const struct dyesub_backend *backend)
 		DEBUG("Environment variables:\n");
 		DEBUG(" DYESUB_DEBUG EXTRA_PID EXTRA_VID EXTRA_TYPE BACKEND SERIAL OLD_URI_SCHEME BACKEND_QUIET\n");
 		DEBUG("CUPS Usage:\n");
-		DEBUG("\tDEVICE_URI=someuri %s job user title num-copies options [ filename ]\n", URI_PREFIX);
+		DEBUG("\tDEVICE_URI=someuri %s job user title num-copies options [ filename ]\n", ptr);
 		DEBUG("\n");
 		DEBUG("Standalone Usage:\n");
-		DEBUG("\t%s\n", URI_PREFIX);
+		DEBUG("\t%s\n", ptr);
 		DEBUG("  [ -D ] [ -G ] [ -f ] [ -v ]\n");
 		DEBUG("  [ backend_specific_args ] \n");
 		DEBUG("  [ -d copies ] \n");
@@ -1072,11 +1095,11 @@ void print_help(const char *argv0, const struct dyesub_backend *backend)
 	}
 
 	/* Scan for all printers for the specified backend */
-	find_and_enumerate(ctx, &list, backend, NULL, ptr, 1, 1, NULL);
+	find_and_enumerate(argv0, &list, backend, NULL, ptr, 1, 1, NULL);
 	libusb_free_device_list(list, 1);
 }
 
-static int parse_cmdstream(struct dyesub_backend *backend, void *backend_ctx, int fd)
+static int parse_cmdstream(const struct dyesub_backend *backend, void *backend_ctx, int fd)
 {
 	FILE *fp = stdin;
 	char line[128];
@@ -1117,13 +1140,11 @@ static int parse_cmdstream(struct dyesub_backend *backend, void *backend_ctx, in
 }
 
 static int handle_input(struct dyesub_backend *backend, void *backend_ctx,
-			const char *fname, char *uri, char *type)
+			const char *fname, const char *uri, const char *type)
 {
 	int ret = CUPS_BACKEND_OK;
-#ifndef _WIN32
 	int i;
-#endif
-	const void *job;
+	const void *jobs[MAX_JOBS_FROM_READ_PARSE];
 	int data_fd = fileno(stdin);
 	int read_page = 0, print_page = 0;
 	struct dyesub_joblist *jlist = NULL;
@@ -1172,16 +1193,17 @@ static int handle_input(struct dyesub_backend *backend, void *backend_ctx,
 	signal(SIGTERM, sigterm_handler);
 #endif
 
-	/* Time for the main processing loop */
-	INFO("Printing started (%d copies)\n", ncopies);
-
 	/* See if it's a CUPS command stream, and if yes, handle it! */
 	if (type && !strcmp("application/vnd.cups-command", type))
 	{
+		INFO("CUPS Command mode\n");
 		ret = parse_cmdstream(backend, backend_ctx, data_fd);
 		goto done;
 	}
 
+	/* Time for the main processing loop */
+	INFO("Printing started\n");
+
 	/* Emit a verbose marker dump */
 	ret = query_markers(backend, backend_ctx, 1);
 	if (ret)
@@ -1189,15 +1211,17 @@ static int handle_input(struct dyesub_backend *backend, void *backend_ctx,
 
 newpage:
 	/* Read in data */
-	job = NULL;
-	if ((ret = backend->read_parse(backend_ctx, &job, data_fd, ncopies))) {
+	for (i = 0 ; i < MAX_JOBS_FROM_READ_PARSE ; i++)
+		jobs[i] = NULL;
+
+	if ((ret = backend->read_parse(backend_ctx, jobs, data_fd, ncopies))) {
 		if (read_page)
 			goto done_multiple;
 		else
 			goto done;
 	}
 
-	if (!job) {
+	if (!jobs[0]) {
 		WARNING("No job returned by backend read_parse?\n");
 		goto newpage;
 	}
@@ -1207,12 +1231,16 @@ newpage:
 		jlist = dyesub_joblist_create(backend, backend_ctx);
 	}
 	if (!jlist) {
-		backend->cleanup_job(job);
+		for (i = 0 ; i < MAX_JOBS_FROM_READ_PARSE ; i++)
+			backend->cleanup_job(jobs[i]);
 		goto done;
 	}
 
-	/* Stick it onto the end of the list */
-	dyesub_joblist_appendjob(jlist, job);
+	/* Stick jobs onto the end of the list */
+	for (i = 0 ; i < MAX_JOBS_FROM_READ_PARSE ; i++) {
+		if (jobs[i])
+			dyesub_joblist_appendjob(jlist, jobs[i]);
+	}
 	read_page++;
 
 	INFO("Parsed page %d (%d copies)\n", read_page, ncopies);
@@ -1251,7 +1279,6 @@ done:
 
 int main (int argc, char **argv)
 {
-	struct libusb_context *ctx = NULL;
 	struct libusb_device **list = NULL;
 
 	struct dyesub_backend *backend = NULL;
@@ -1264,12 +1291,19 @@ int main (int argc, char **argv)
 	int found = -1;
 	int jobid = 0;
 
-	int stats_only = 0;
 	char *uri;
 	char *type;
 	const char *fname = NULL;
 	char *use_serno = NULL;
 	const char *backend_str = NULL;
+	const char *argv0;
+
+	/* Work out path-less executable name */
+	argv0 = strrchr(argv[0], '/');
+	if (argv0)
+		argv0++;
+	else
+		argv0 = argv[0];
 
 	logger = stderr;
 
@@ -1288,8 +1322,10 @@ int main (int argc, char **argv)
 		extra_type = atoi(getenv("EXTRA_TYPE"));
 	if (getenv("BACKEND"))
 		backend_str = getenv("BACKEND");
+	else if (getenv("DYESUB_BACKEND"))
+		backend_str = getenv("DYESUB_BACKEND");
 	if (getenv("FAST_RETURN"))
-		fast_return++;
+		fast_return = atoi(getenv("FAST_RETURN"));
 	if (getenv("MAX_XFER_SIZE"))
 		max_xfer_size = atoi(getenv("MAX_XFER_SIZE"));
 	if (getenv("XFER_TIMEOUT"))
@@ -1306,19 +1342,19 @@ int main (int argc, char **argv)
 		exit(1);
 	}
 
-	if (stats_only)
+	if (stats_only && !dyesub_debug)
 		quiet = 1;
 
 	DEBUG("Multi-Call Dye-sublimation CUPS Backend version %s\n",
 	      BACKEND_VERSION);
-	DEBUG("Copyright 2007-2020 Solomon Peachy\n");
+	DEBUG("Copyright 2007-2025 Solomon Peachy\n");
 	DEBUG("This free software comes with ABSOLUTELY NO WARRANTY! \n");
 	DEBUG("Licensed under the GNU GPL.  Run with '-G' for more details.\n");
 	DEBUG("\n");
 
 	use_serno = getenv("SERIAL");
 	uri = getenv("DEVICE_URI");  /* CUPS backend mode! */
-	type = getenv("FINAL_CONTENT_TYPE"); /* CUPS content type -- ie raster or command */
+	type = getenv("CONTENT_TYPE"); /* CUPS content type -- ie raster or command */
 
 	if (uri && strlen(uri)) {  /* CUPS backend mode */
 		int base = optind; /* ie 1 */
@@ -1328,12 +1364,13 @@ int main (int argc, char **argv)
 		}
 		if (argv[base])
 			jobid = atoi(argv[base]);
-		if (argv[base + 3])
-			ncopies = atoi(argv[base + 3]);
-		if (argc > 6)
+		if (argc > 6) {
 			fname = argv[base + 5];
-		else
+			if (argv[base + 3])
+				ncopies = atoi(argv[base + 3]);
+		} else {
 			fname = "-";
+		}
 
 		/* Figure out backend based on URI */
 		{
@@ -1376,19 +1413,15 @@ int main (int argc, char **argv)
 			}
 		}
 
-		/* Always enable fast return in CUPS mode */
-		fast_return++;
+		/* Enable fast return in CUPS if it's not supplied */
+		if (!getenv("FAST_RETURN"))
+			fast_return++;
 
 	} else {  /* Standalone mode */
 
 		/* Try to guess backend from executable name */
-		if (!backend_str) {
-			backend_str = strrchr(argv[0], '/');
-			if (backend_str)
-				backend_str++;
-			else
-				backend_str = argv[0];
-		}
+		if (!backend_str)
+			backend_str = argv0;
 
 		srand(getpid());
 		jobid = rand();
@@ -1414,7 +1447,7 @@ int main (int argc, char **argv)
 #endif
 
 	/* Libusb setup */
-	ret = libusb_init(&ctx);
+	ret = libusb_init(NULL);
 	if (ret) {
 		ERROR("Failed to initialize libusb (%d)\n", ret);
 		ret = CUPS_BACKEND_RETRY_CURRENT;
@@ -1423,7 +1456,7 @@ int main (int argc, char **argv)
 
 	/* If we don't have a valid backend, print help and terminate */
 	if (!backend && !stats_only) {
-		print_help(argv[0], NULL); // probes all devices
+		print_help(argv0, NULL); // probes all devices
 		ret = CUPS_BACKEND_OK;
 		goto done;
 	}
@@ -1431,20 +1464,24 @@ int main (int argc, char **argv)
 	/* If we're in standalone mode, print help only if no args */
 	if ((!uri || !strlen(uri)) && !stats_only) {
 		if (argc < 2) {
-			print_help(argv[0], backend); // probes all devices
+			print_help(argv0, backend); // probes all devices
 			ret = CUPS_BACKEND_OK;
 			goto done;
 		}
 	}
 
 	/* Enumerate devices */
-	found = find_and_enumerate(ctx, &list, backend, use_serno, backend_str, 0, NUM_CLAIM_ATTEMPTS, &conn);
+	STATE("+connecting-to-device\n");
+
+	found = find_and_enumerate(argv0, &list, backend, use_serno, backend_str, 0, NUM_CLAIM_ATTEMPTS, &conn);
 
 	if (found == -1) {
 		ERROR("Printer open failure (No matching printers found!)\n");
+		STATE("+offline-report\n");
 		ret = CUPS_BACKEND_RETRY;
 		goto done;
 	}
+	STATE("-offline-report\n");
 
 	if (test_mode) {
 		WARNING("**** TEST MODE %d!\n", test_mode);
@@ -1489,6 +1526,8 @@ int main (int argc, char **argv)
 	}
 
 bypass:
+	STATE("-connecting-to-device\n");
+
 	/* Initialize backend */
 	DEBUG("Initializing '%s' backend (version %s)\n",
 	      backend->name, backend->version);
@@ -1503,9 +1542,13 @@ bypass:
 
 		conn.type = lookup_printer_type(backend,
 						desc.idVendor, desc.idProduct);
+		conn.usb_vid = desc.idVendor;
+		conn.usb_pid = desc.idProduct;
 	} else {
 		conn.type = lookup_printer_type(backend,
 						extra_vid, extra_pid);
+		conn.usb_vid = extra_vid;
+		conn.usb_pid = extra_pid;
 	}
 
 	if (conn.type <= P_UNKNOWN) {
@@ -1522,8 +1565,6 @@ bypass:
 		goto done_claimed;
 	}
 
-//	STATE("+org.gutenprint.attached-to-device\n");
-
 	/* Dump stats only */
 	if (stats_only && backend->query_stats) {
 		struct printerstats stats;
@@ -1541,6 +1582,14 @@ bypass:
 		goto done_claimed;
 	}
 
+	PPD("StpUsbBackend=\"%s\"\n", backend_str ? backend_str : backend->name);
+	PPD("StpUsbVid=%04x\n", conn.usb_vid);
+	PPD("StpUsbPid=%04x\n", conn.usb_pid);
+	if (test_mode < TEST_MODE_NOATTACH) {
+		PPD("StpUsbBus=%03d\n", conn.bus_num);
+		PPD("StpUsbPort=%03d\n", conn.port_num);
+	}
+
 	if (!uri || !strlen(uri)) {
 		if (backend->cmdline_arg(backend_ctx, argc, argv))
 			goto done_claimed;
@@ -1561,18 +1610,19 @@ done_close:
 		libusb_close(conn.dev);
 done:
 
+	STATE("-connecting-to-device\n");
+
 	if (backend && backend_ctx) {
 		if (backend->teardown)
 			backend->teardown(backend_ctx);
 		else
 			generic_teardown(backend_ctx);
-//		STATE("-org.gutenprint.attached-to-device");
 	}
 
 	if (list)
 		libusb_free_device_list(list, 1);
 
-	libusb_exit(ctx);
+	libusb_exit(NULL);
 
 	return ret;
 }
@@ -1673,18 +1723,20 @@ minimal:
 	if (full && getenv("DEVICE_URI")) {
 		for (i = 0 ; i < marker_count ; i++) {
 			PPD("StpMediaID%d=%d\n", i, markers[i].numtype);
+			PPD("StpMediaName%d=\"%s\"\n", i, markers[i].name);
 		}
 	}
 }
 
-int dyesub_read_file(const char *filename, void *databuf, int datalen,
-		     int *actual_len)
+int dyesub_read_file2(const char *filename, void *databuf, int datalen,
+		      int *actual_len, int fail_ok)
 {
 	int len;
 	int fd = open(filename, O_RDONLY);
 
 	if (fd < 0) {
-		ERROR("Unable to open '%s'\n", filename);
+		if (!fail_ok)
+			ERROR("Unable to open '%s'\n", filename);
 		return CUPS_BACKEND_FAILED;
 	}
 	len = read(fd, databuf, datalen);
@@ -1756,7 +1808,7 @@ struct dyesub_joblist *dyesub_joblist_create(const struct dyesub_backend *backen
 	list->num_entries = 0;
 
 	if (collate)
-		list->copies = ncopies;
+		list->copies = ncopies; // XXX
 	else
 		list->copies = 1;
 
@@ -1910,6 +1962,90 @@ const void *dyesub_joblist_popjob(struct dyesub_joblist *list)
 	return NULL;
 }
 
+#include "backend_panodata.h"
+#define PROCESS_PIXEL(__corr, __offset)				\
+	data[(r * cols * 3) + c + __offset] = 255 - ((255 - (double)data[(r * cols *3) + c +__offset]) * (__corr))
+
+static void dyesub_pano_process_rgb8(const struct dnp_panodata *pano,
+				     uint8_t *data, int lh, int rh,
+				     uint16_t cols, uint16_t rows,
+				     uint16_t overlap, uint16_t pad_rows)
+{
+	/* Skip over start margin in source */
+	data += pad_rows * cols * 3;
+
+	for (int r = 0 ; r < rows ; r++) {
+		if (rh && r < overlap) {
+			const struct panodata_row *rhc;
+			int i, c;
+			int row = (overlap - r);
+			for (i = 0 ; i < pano->elements-1 ; i++) {
+				if (row >= pano->rows[i].start_row && row < pano->rows[i+1].start_row)
+					break;
+			}
+			rhc = &pano->rows[i];
+			for (c = 0 ; c < cols ; c++) {
+				PROCESS_PIXEL(rhc->rhYMC[2],0); /* R/C */
+				PROCESS_PIXEL(rhc->rhYMC[1],1); /* G/M */
+				PROCESS_PIXEL(rhc->rhYMC[0],2); /* B/Y */
+			}
+		} else if (lh && (rows - r) < overlap) {
+			const struct panodata_row *lhc;
+			int i, c;
+			int row = (rows -r);
+			for (i = 0 ; i < pano->elements-1 ; i++) {
+				if (row >= pano->rows[i].start_row && row < pano->rows[i+1].start_row)
+					break;
+			}
+			lhc = &pano->rows[i];
+			for (c = 0 ; c < cols ; c++) {
+				PROCESS_PIXEL(lhc->rhYMC[2],0); /* R/C */
+				PROCESS_PIXEL(lhc->rhYMC[1],1); /* G/M */
+				PROCESS_PIXEL(lhc->rhYMC[0],2); /* B/Y */
+			}
+		}
+	}
+}
+
+void dyesub_pano_split_rgb8(const uint8_t *src, uint16_t cols, uint8_t numpanels,
+			    uint16_t overlap_rows, uint16_t pad_rows,
+			    uint16_t *panel_rows, uint8_t **panels)
+{
+	int i = 0;
+
+	INFO("Splitting job into %d panel continuous panorama\n", numpanels);
+
+	/* Skip over start margin in source */
+	src += pad_rows * cols * 3;
+
+	for (i = 0 ; i < numpanels ; i++) {
+		int lh = (i < (numpanels -1));
+		int rh = (i > 0);
+		uint8_t *out = panels[i];
+
+		/* Fill start margin with white */
+		if (pad_rows) {
+			memset(out, 0xff, pad_rows * cols * 3);
+			out += pad_rows * cols * 3;
+		}
+
+		/* Copy over panel data */
+		memcpy(out, src, panel_rows[i] * cols * 3);
+		src += (panel_rows[i] - overlap_rows) * cols * 3; /* Factor in overlap */
+		out += panel_rows[i] * cols * 3;
+
+		/* Fill end margin with white */
+		if (pad_rows) {
+			memset(out, 0xff, pad_rows * cols * 3);
+			out += pad_rows * cols * 3;
+		}
+
+		dyesub_pano_process_rgb8(&panodata, panels[i], lh, rh,
+					 cols, panel_rows[i],
+					 overlap_rows, pad_rows);
+	}
+}
+
 int dyesub_joblist_canwait(struct dyesub_joblist *list)
 {
 	if (list->num_entries == DYESUB_MAX_JOB_ENTRIES)
@@ -1928,6 +2064,10 @@ int dyesub_joblist_print(const struct dyesub_joblist *list, int *pagenum)
 
 	for (i = 0 ; i < list->copies ; i++) {
 		for (j = 0 ; j < list->num_entries ; j++) {
+			int wait_on_return = 0;
+			if (i == (list->copies - 1) && j == (list->num_entries -1))
+				wait_on_return = !fast_return; /* only wait on the final copy/page. */
+
 			if (list->entries[j]) {
 				int copies = ((const struct dyesub_job_common *)(list->entries[j]))->copies;
 
@@ -1938,7 +2078,7 @@ int dyesub_joblist_print(const struct dyesub_joblist *list, int *pagenum)
 				/* Print this page */
 				if (test_mode < TEST_MODE_NOPRINT ||
 				    list->backend->flags & BACKEND_FLAG_DUMMYPRINT) {
-					ret = list->backend->main_loop(list->ctx, list->entries[j]);
+					ret = list->backend->main_loop(list->ctx, list->entries[j], wait_on_return);
 					if (ret)
 						return ret;
 				}
