@@ -1,11 +1,11 @@
 /*
  *   CUPS Backend common code
  *
- *   (c) 2013-2020 Solomon Peachy <pizza@shaftnet.org>
+ *   (c) 2013-2024 Solomon Peachy <pizza@shaftnet.org>
  *
  *   The latest version of this program can be found at:
  *
- *     http://git.shaftnet.org/cgit/selphy_print.git
+ *     https://git.shaftnet.org/gitea/slp/selphy_print.git
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the Free
@@ -18,9 +18,7 @@
  *   for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
- *
- *          [http://www.gnu.org/licenses/gpl-2.0.html]
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  *   SPDX-License-Identifier: GPL-2.0+
  *
@@ -39,6 +37,9 @@
 
 #include <libusb.h>
 
+#pragma GCC diagnostic push
+#pragma GCC diagnostic   ignored "-Wcast-qual"
+
 /* For Integration into gutenprint */
 #if defined(HAVE_CONFIG_H)
 #include <config.h>
@@ -58,9 +59,9 @@
 #define DEBUG( ... ) do { if (!quiet) fprintf(logger, "DEBUG: " __VA_ARGS__ ); } while(0)
 #define DEBUG2( ... ) do { if (!quiet) fprintf(logger, __VA_ARGS__ ); } while(0)
 #define INFO( ... )  do { if (!quiet) fprintf(logger, "INFO: " __VA_ARGS__ ); } while(0)
-#define WARNING( ... )  do { fprintf(logger, "WARNING: " __VA_ARGS__ ); } while(0)
+#define WARNING( ... )  do { if (!quiet) fprintf(logger, "WARNING: " __VA_ARGS__ ); } while(0)
 #define ERROR( ... ) do { fprintf(logger, "ERROR: " __VA_ARGS__ ); sleep(1); } while (0)
-#define PPD( ... ) do { fprintf(logger, "PPD: " __VA_ARGS__ ); sleep(1); } while (0)
+#define PPD( ... ) do { fprintf(logger, "PPD: " __VA_ARGS__ ); } while (0)
 
 #if (__BYTE_ORDER == __LITTLE_ENDIAN)
 #define le16_to_cpu(__x) __x
@@ -100,7 +101,7 @@ enum {
 	P_UNKNOWN = 0,
 	P_CITIZEN_CW01,
 	P_CITIZEN_OP900II,
-	P_CP_XXX,
+	P_CPGENERIC,
 	P_CP10,
 	P_CP790,
 	P_CP900,
@@ -112,15 +113,24 @@ enum {
 	P_DNP_DS820,
 	P_DNP_DSRX1,
 	P_DNP_QW410,
+	P_DNP_DSX80,
 	P_ES1,
 	P_ES2_20,
 	P_ES3_30,
 	P_ES40,
 	P_FUJI_ASK300,
+	P_FUJI_ASK500,
+	P_HITI_CS2XX,
+	P_HITI_310,
+	P_HITI_320,
+	P_HITI_461,
 	P_HITI_51X,
-	P_HITI_52X,
+	P_HITI_520,
+	P_HITI_525,
+	P_HITI_530,
 	P_HITI_720,
 	P_HITI_750,
+	P_HITI_826,
 	P_HITI_910,
 	P_KODAK_1400_805,
 	P_KODAK_305,
@@ -130,19 +140,25 @@ enum {
 	P_KODAK_6900,
 	P_KODAK_7000,
 	P_KODAK_701X,
+	P_KODAK_8800,
 	P_KODAK_8810,
 	P_MAGICARD,
+	P_NDC,
+	P_MITSU_9500S,
 	P_MITSU_9550,
 	P_MITSU_9550S,
 	P_MITSU_9600,
+	P_MITSU_9600S,
 	P_MITSU_9800,
 	P_MITSU_9800S,
 	P_MITSU_9810,
+	P_MITSU_9820S,
 	P_MITSU_D70X,
 	P_MITSU_D80,
 	P_MITSU_D90,
 	P_MITSU_K60,
 	P_MITSU_M1,
+	P_MITSU_W5000,
 	P_MITSU_P93D,
 	P_MITSU_P95D,
 	P_MITSU_CP30D,
@@ -154,9 +170,11 @@ enum {
 	P_SHINKO_S6245,
 	P_SONY_UPCR10,
 	P_SONY_UPCR20L,
+	P_SONY_UPD711,
 	P_SONY_UPD895,
 	P_SONY_UPD897,
 	P_SONY_UPD898,
+	P_SONY_UP9x1,
 	P_SONY_UPDR150,
 	P_SONY_UPDR80,
 	P_END,
@@ -202,6 +220,12 @@ struct dyesub_connection {
 	uint8_t iface;
 	uint8_t altset;
 
+	/* to make our lives easier later */
+	uint8_t bus_num;
+	uint8_t port_num;
+	uint16_t usb_vid;
+	uint16_t usb_pid;
+
 	// TODO:  mutex/lock
 
 	int type; /* P_XXXX */
@@ -217,14 +241,28 @@ struct dyesub_joblist {
 	int copies;
 	const void *entries[DYESUB_MAX_JOB_ENTRIES];
 };
+#define MAX_JOBS_FROM_READ_PARSE 3
 
-/* This should be the start of every per-printer job struct! */
+/* This MUST be the start of every per-printer job struct! */
 struct dyesub_job_common {
 	size_t jobsize;
 	int copies;
 	int can_combine;
 };
 
+/* Panorama stuff */
+struct panodata_row {
+        uint16_t start_row;
+        double rhYMC[3];
+        double lhYMC[3];
+};
+
+#define DNP_PANO_MAXROWS 64
+struct dnp_panodata {
+        uint16_t elements;
+        struct panodata_row rows[DNP_PANO_MAXROWS];
+};
+
 /* Exported functions */
 int send_data(struct dyesub_connection *conn, const uint8_t *buf, int len);
 int read_data(struct dyesub_connection *conn,
@@ -235,8 +273,11 @@ void dump_markers(const struct marker *markers, int marker_count, int full);
 void print_license_blurb(void);
 void print_help(const char *argv0, const struct dyesub_backend *backend);
 
-int dyesub_read_file(const char *filename, void *databuf, int datalen,
-		     int *actual_len);
+int dyesub_read_file2(const char *filename, void *databuf, int datalen,
+		      int *actual_len, int fail_ok);
+
+#define dyesub_read_file(__fname, __databuf, __datalen, __actual_len) \
+	dyesub_read_file2(__fname, __databuf, __datalen, __actual_len, 0)
 
 uint16_t uint16_to_packed_bcd(uint16_t val);
 uint32_t packed_bcd_to_uint32(const char *in, int len);
@@ -259,26 +300,39 @@ int dyesub_joblist_canwait(struct dyesub_joblist *list);
 #define BACKEND_FLAG_BADISERIAL 0x00000001
 #define BACKEND_FLAG_DUMMYPRINT 0x00000002
 
+void dyesub_pano_split_rgb8(const uint8_t *src, uint16_t cols, uint8_t numpanels,
+			    uint16_t overlap_rows, uint16_t pad_rows,
+			    uint16_t *panel_rows, uint8_t **panels);
+
 /* Backend Functions */
 struct dyesub_backend {
+	/* Required properties */
 	const char *name;
 	const char *version;
 	const char **uri_prefixes;
+	const struct device_id *devices;
+
+	/* Optional properties */
 	const uint32_t flags;
-	void (*cmdline_usage)(void);  /* Optional */
+
+	/* Required functions */
 	void *(*init)(void);
 	int  (*attach)(void *ctx, struct dyesub_connection *conn, uint8_t jobid);
-	void (*teardown)(void *ctx);
 	int  (*cmdline_arg)(void *ctx, int argc, char **argv);
 	int  (*read_parse)(void *ctx, const void **job, int data_fd, int copies);
+	int  (*main_loop)(void *ctx, const void *job, int wait_on_return);
 	void (*cleanup_job)(const void *job);
+
+	/* Required if backend implemnets job combining */
 	void *(*combine_jobs)(const void *job1, const void *job2);
 	int  (*job_polarity)(void *ctx);
-	int  (*main_loop)(void *ctx, const void *job);
-	int  (*query_serno)(struct dyesub_connection *conn, char *buf, int buf_len); /* Optional */
+
+	/* Optional stuff */
+	void (*teardown)(void *ctx);
+	void (*cmdline_usage)(void);
+	int  (*query_serno)(struct dyesub_connection *conn, char *buf, int buf_len);
 	int  (*query_markers)(void *ctx, struct marker **markers, int *count);
-	int  (*query_stats)(void *ctx, struct printerstats *stats); /* Optional */
-	const struct device_id devices[];
+	int  (*query_stats)(void *ctx, struct printerstats *stats);
 };
 
 /* Global data */
@@ -294,6 +348,7 @@ extern int test_mode;
 extern int quiet;
 extern const char *corrtable_path;
 extern FILE *logger;
+extern int stats_only;
 
 enum {
 	TEST_MODE_NONE = 0,
@@ -364,8 +419,10 @@ extern const struct dyesub_backend BACKEND;
 #define DL_CLOSE(__x) do {} while(0)
 #define DL_EXIT()     do {} while(0)
 #endif
-#ifdef _WIN32
+#if defined(_WIN32)
 #define DLL_SUFFIX ".dll"
+#elif defined(__APPLE__)
+#define DLL_SUFFIX ".dylib"
 #else
 #define DLL_SUFFIX ".so"
 #endif
