@@ -1,11 +1,11 @@
 /*
- *   Kodak 605 Photo Printer CUPS backend -- libusb-1.0 version
+ *   Kodak 605 Photo Printer CUPS backend
  *
- *   (c) 2013-2020 Solomon Peachy <pizza@shaftnet.org>
+ *   (c) 2013-2024 Solomon Peachy <pizza@shaftnet.org>
  *
  *   The latest version of this program can be found at:
  *
- *     http://git.shaftnet.org/cgit/selphy_print.git
+ *     https://git.shaftnet.org/gitea/slp/selphy_print.git
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the Free
@@ -18,9 +18,7 @@
  *   for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
- *
- *          [http://www.gnu.org/licenses/gpl-2.0.html]
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  *   SPDX-License-Identifier: GPL-2.0+
  *
@@ -31,12 +29,6 @@
 #include "backend_common.h"
 #include "backend_sinfonia.h"
 
-#define USB_VID_KODAK       0x040A
-#define USB_PID_KODAK_605   0x402E
-#define USB_PID_KODAK_7000  0x4035
-#define USB_PID_KODAK_7010  0x4037
-#define USB_PID_KODAK_7015  0x4038
-
 /* Media structure */
 struct kodak605_media_list {
 	struct sinfonia_status_hdr hdr;
@@ -85,55 +77,55 @@ struct kodak605_status {
 
 static const struct sinfonia_param ek7000_params[] =
 {
-	{ 0x01, "Unknown_01" }, // 00000001
-	{ 0x11, "Unknown_11" }, // 00000001
-	{ 0x12, "Matte Gloss" }, // 00000069
-	{ 0x13, "Matte Degloss Black" }, // 000000c3
-	{ 0x14, "Matte Degloss White" }, // 000000cd
-	{ 0x21, "Exit Speed With Sorter 4x6" }, // 000003e8
-	{ 0x22, "Exit Speed With Sorter 8x6" }, // 0000041a
-	{ 0x23, "Exit Speed With Backprinting" }, // 00000152
-	{ 0x24, "Exit Speed Without PPAC 4x6" }, // 0000044c
-	{ 0x25, "Exit Speed Without PPAC 8x6" }, // 0000044c
+	{ PARAM_UNK_01, "Unknown_01" },
+	{ PARAM_UNK_11, "Unknown_11" },
+	{ PARAM_UNK_12, "Matte Gloss" },
+	{ PARAM_UNK_13, "Matte Degloss Black" },
+	{ PARAM_UNK_14, "Matte Degloss White" },
+	{ PARAM_UNK_21, "Exit Speed With Sorter 4x6" },
+	{ PARAM_UNK_22, "Exit Speed With Sorter 8x6" },
+	{ PARAM_UNK_23, "Exit Speed With Backprinting" },
+	{ PARAM_UNK_24, "Exit Speed Without PPAC 4x6" },
+	{ PARAM_UNK_25, "Exit Speed Without PPAC 8x6" },
 
-	{ 0x2f, "Unknown_2f" }, // 00000320
-	{ 0x41, "Unknown_41" }, // 0000006d
-	{ 0x42, "Unknown_42" }, // 00000051
-	{ 0x43, "Unknown_43" }, // 0000003b
-	{ 0x44, "Unknown_44" }, // 00000082
-	{ 0x45, "Unknown_45" }, // 00000000
-	{ 0x46, "Unknown_46" }, // 00000000
-	{ 0x47, "Unknown_47" }, // 00000028
-	{ 0x48, "Unknown_48" }, // 00000002
-	{ 0x81, "Unknown_81" }, // ffffffff
+	{ PARAM_UNK_2F, "Unknown_2f" },
+	{ PARAM_UNK_41, "Unknown_41" },
+	{ PARAM_UNK_42, "Unknown_42" },
+	{ PARAM_UNK_43, "Unknown_43" },
+	{ PARAM_UNK_44, "Unknown_44" },
+	{ PARAM_UNK_45, "Unknown_45" },
+	{ PARAM_UNK_46, "Unknown_46" },
+	{ PARAM_UNK_47, "Unknown_47" },
+	{ PARAM_UNK_48, "Unknown_48" },
+	{ PARAM_UNK_81, "Unknown_81" },
 
-	{ 0x82, "Unknown_82" }, // fffffffe
-	{ 0x83, "Unknown_83" }, // ffffffee
-	{ 0x84, "Unknown_84" }, // 00000001
-	{ 0x91, "Unknown_91" }, // 0000006c
-	{ 0x92, "Unknown_92" }, // 00000077
-	{ 0x93, "Unknown_93" }, // 00000067
-	{ 0x94, "Unknown_94" }, // 00000076
-	{ 0xa0, "Unknown_a0" }, // 00000005
-	{ 0xa1, "Unknown_a1" }, // 00000000
-	{ 0xa2, "Unknown_a2" }, // 00000010
+	{ PARAM_UNK_82, "Unknown_82" },
+	{ PARAM_UNK_83, "Unknown_83" },
+	{ PARAM_UNK_84, "Unknown_84" },
+	{ PARAM_UNK_91, "Unknown_91" },
+	{ PARAM_UNK_92, "Unknown_92" },
+	{ PARAM_UNK_93, "Unknown_93" },
+	{ PARAM_UNK_94, "Unknown_94" },
+	{ PARAM_UNK_A0, "Unknown_a0" },
+	{ PARAM_UNK_A1, "Unknown_a1" },
+	{ PARAM_UNK_A2, "Unknown_a2" },
 
-	{ 0xa3, "Unknown_a3" }, // 0000003b
-	{ 0xa4, "Unknown_a4" }, // 0000003b
-	{ 0xa5, "Thermal Protect Lamination" }, // 0000003e
-	{ 0xa6, "Unknown_a6" }, // 00000001
-	{ 0xa7, "Unknown_a7" }, // 00000014
-	{ 0xa8, "Unknown_a8" }, // 00000001
-	{ 0xa9, "Unknown_a9" }, // ffffffff
-	{ 0xc1, "Unknown_c1" }, // 00000002
-	{ 0xc2, "Unknown_c2" }, // 000000c8
-	{ 0xc3, "Unknown_c3" }, // 000000c8
+	{ PARAM_UNK_A3, "Unknown_a3" },
+	{ PARAM_UNK_A4, "Unknown_a4" },
+	{ PARAM_UNK_A5, "Thermal Protect Lamination" },
+	{ PARAM_UNK_A6, "Unknown_a6" },
+	{ PARAM_UNK_A7, "Unknown_a7" },
+	{ PARAM_UNK_A8, "Unknown_a8" },
+	{ PARAM_UNK_A9, "Unknown_a9" },
+	{ PARAM_UNK_C1, "Unknown_c1" },
+	{ PARAM_UNK_C2, "Unknown_c2" },
+	{ PARAM_UNK_C3, "Unknown_c3" },
 
-	{ 0xc4, "Unknown_c4" }, // 00000200
-	{ 0xf1, "Unknown_f1" }, // 00000068
-	{ 0xf2, "Unknown_f2" }, // 00000068
-	{ 0xf3, "Unknown_f3" }, // 00000094
-	{ 0xf4, "Unknown_f4" }, // 00000068
+	{ PARAM_UNK_C4, "Unknown_c4" },
+	{ PARAM_UNK_F1, "Unknown_f1" },
+	{ PARAM_UNK_F2, "Unknown_f2" },
+	{ PARAM_UNK_F3, "Unknown_f3" },
+	{ PARAM_UNK_F4, "Unknown_f4" },
 };
 #define ek7000_params_num (sizeof(ek7000_params) / sizeof(struct sinfonia_param))
 
@@ -151,8 +143,60 @@ struct kodak605_ctx {
 	struct marker marker;
 };
 
-/* Note this is for Kodak 7000-series only! */
-static const char *error_codes(uint8_t major, uint8_t minor)
+static const char *error_codes_605(uint8_t major, uint8_t minor)
+{
+	switch(major) {
+	case 0x01: /* "Controller Error" */
+		switch(minor) {
+		case 0x0c:
+		default:
+			return "Controller: Unknown";
+		}
+	case 0x02: /* "Mechanical Error" */
+		switch (minor) {
+		default:
+			return "Mechanical: Unknown";
+		}
+	case 0x03: /* "Sensor Error" */
+		switch (minor) {
+		default:
+			return "Sensor: Unknown";
+		}
+	case 0x04: /* "Temperature Sensor Error" */
+		switch (minor) {
+		default:
+			return "Temp Sensor: Unknown";
+		}
+	case 0x5: /* "Paper Jam" */
+		switch (minor) {
+		case 0x0c:
+		case 0x17:
+		case 0x2e:
+		case 0x52:
+		case 0x53:
+		default:
+			return "Paper Jam: Unknown";
+		}
+	case 0x06: /* User Error */
+		switch (minor) {
+		case 0x01:
+			return "Top Cover Open";
+		case 0x02:
+			return "Paper Cover Open";
+		case 0x05:
+			return "Ribbon Empty";
+		case 0x08:
+			return "Paper Empty";
+		default:
+			return "User: Unknown";
+		}
+	default:
+		return "Unknown";
+	}
+}
+
+
+static const char *error_codes_7000(uint8_t major, uint8_t minor)
 {
 	switch(major) {
 	case 0x01: /* "Controller Error" */
@@ -359,14 +403,22 @@ static void *kodak605_init(void)
 	return ctx;
 }
 
+static void kodak605_teardown(void *vctx)
+{
+	struct kodak605_ctx *ctx = vctx;
+	free(ctx->media);
+	free(ctx);
+}
+
 static int kodak605_attach(void *vctx, struct dyesub_connection *conn, uint8_t jobid)
 {
 	struct kodak605_ctx *ctx = vctx;
 
 	ctx->dev.conn = conn;
-	ctx->dev.error_codes = &error_codes;
+	ctx->dev.error_codes = error_codes_605;
 
 	if (ctx->dev.conn->type != P_KODAK_605) {
+		ctx->dev.error_codes = &error_codes_7000;
 		ctx->dev.params = ek7000_params;
 		ctx->dev.params_count = ek7000_params_num;
 	}
@@ -385,7 +437,7 @@ static int kodak605_attach(void *vctx, struct dyesub_connection *conn, uint8_t j
 	} else {
 		int media_code = KODAK6_MEDIA_6TR2;
 		if (getenv("MEDIA_CODE"))
-			media_code = atoi(getenv("MEDIA_CODE"));
+			media_code = strtol(getenv("MEDIA_CODE"), NULL, 16);
 
 		ctx->media->type = media_code;
 	}
@@ -422,16 +474,16 @@ static int kodak605_read_parse(void *vctx, const void **vjob, int data_fd, int c
 		return ret;
 	}
 
-	/* Printer handles generating copies.  Use larger of our options */
-	if (le16_to_cpu(job->jp.copies) < (uint16_t)copies)
-		job->jp.copies = cpu_to_le16(copies);
+	/* Use larger of our copy counts */
+	if (job->common.copies < copies)
+		job->common.copies = copies;
 
 	*vjob = job;
 
 	return CUPS_BACKEND_OK;
 }
 
-static int kodak605_main_loop(void *vctx, const void *vjob) {
+static int kodak605_main_loop(void *vctx, const void *vjob, int wait_for_return) {
 	struct kodak605_ctx *ctx = vctx;
 
 	struct kodak605_status sts;
@@ -482,7 +534,7 @@ static int kodak605_main_loop(void *vctx, const void *vjob) {
 			      sts.hdr.result, sts.hdr.error,
 			      sinfonia_error_str(sts.hdr.error),
 			      sts.hdr.printer_major, sts.hdr.printer_minor,
-			      error_codes(sts.hdr.printer_major, sts.hdr.printer_minor));
+			      ctx->dev.error_codes(sts.hdr.printer_major, sts.hdr.printer_minor));
 			return CUPS_BACKEND_FAILED;
 		}
 
@@ -564,7 +616,7 @@ static int kodak605_main_loop(void *vctx, const void *vjob) {
 	hdr.jobid = ctx->jobid;
 	hdr.rows = cpu_to_le16(job->jp.rows);
 	hdr.columns = cpu_to_le16(job->jp.columns);
-	hdr.copies = cpu_to_le16(job->jp.copies);
+	hdr.copies = cpu_to_le16(job->common.copies);
 	hdr.media = job->jp.media;
 	hdr.oc_mode = job->jp.oc_mode;
 	hdr.method = job->jp.method;
@@ -591,7 +643,7 @@ retry_print:
 			ERROR("Result: %02x Error: %02x (%02x %02x = %s)\n",
 			      sts.hdr.result, sts.hdr.error,
 			      sts.hdr.printer_major, sts.hdr.printer_minor,
-			      error_codes(sts.hdr.printer_major, sts.hdr.printer_minor));
+			      ctx->dev.error_codes(sts.hdr.printer_major, sts.hdr.printer_minor));
 
 			return CUPS_BACKEND_FAILED;
 		}
@@ -622,7 +674,7 @@ retry_print:
 			     sts.hdr.result, sts.hdr.error,
 			     sinfonia_error_str(sts.hdr.error),
 			     sts.hdr.printer_major, sts.hdr.printer_minor,
-			     error_codes(sts.hdr.printer_major, sts.hdr.printer_minor));
+			     ctx->dev.error_codes(sts.hdr.printer_major, sts.hdr.printer_minor));
 			return CUPS_BACKEND_STOP;
 		}
 
@@ -639,7 +691,7 @@ retry_print:
 		if (sts.hdr.status == STATUS_READY)
 			break;
 
-		if (fast_return) {
+		if (!wait_for_return) {
 			INFO("Fast return mode enabled.\n");
 			break;
 		}
@@ -656,7 +708,7 @@ static void kodak605_dump_status(struct kodak605_ctx *ctx, struct kodak605_statu
 	INFO("Error: %02x (%s) %02x/%02x = %s\n",
 	     sts->hdr.error, sinfonia_error_str(sts->hdr.error),
 	     sts->hdr.printer_major, sts->hdr.printer_minor,
-	     error_codes(sts->hdr.printer_major, sts->hdr.printer_minor));
+	     ctx->dev.error_codes(sts->hdr.printer_major, sts->hdr.printer_minor));
 
 	INFO("Bank 1: %s - Job %03u @ %03u/%03u\n",
 	     sinfonia_bank_statuses(sts->b1_sts), sts->b1_id,
@@ -700,7 +752,7 @@ static void kodak605_dump_mediainfo(struct kodak605_media_list *media)
 
 	DEBUG("Legal print sizes:\n");
 	for (i = 0 ; i < media->count ; i++) {
-		DEBUG("\t%d: %ux%u (%x)\n", i,
+		DEBUG("\t%d: %ux%u (%02x)\n", i,
 		      media->entries[i].columns,
 		      media->entries[i].rows,
 		      media->entries[i].code);
@@ -893,14 +945,25 @@ static const char *kodak605_prefixes[] = {
 	NULL,
 };
 
+static const struct device_id kodak605_devices[] = {
+	{ 0x040a, 0x402e, P_KODAK_605, "Kodak", "kodak-605"},   /* Shinko CHC-S1545-5A */
+	{ 0x040a, 0x4035, P_KODAK_7000, "Kodak", "kodak-7000"}, /* Shinko CHC-S1645-5A */
+	{ 0x040a, 0x4037, P_KODAK_701X, "Kodak", "kodak-7010"}, /* Shinko CHC-S1645-5B */
+	{ 0x040a, 0x4038, P_KODAK_701X, "Kodak", "kodak-7015"}, /* Shinko CHC-S1645-5C */
+	{ 0, 0, 0, NULL, NULL}
+};
+
+
 /* Exported */
 const struct dyesub_backend kodak605_backend = {
 	.name = "Kodak 605/70xx",
-	.version = "0.55" " (lib " LIBSINFONIA_VER ")",
+	.version = "0.57" " (lib " LIBSINFONIA_VER ")",
 	.uri_prefixes = kodak605_prefixes,
+	.devices = kodak605_devices,
 	.cmdline_usage = kodak605_cmdline,
 	.cmdline_arg = kodak605_cmdline_arg,
 	.init = kodak605_init,
+	.teardown = kodak605_teardown,
 	.attach = kodak605_attach,
 	.cleanup_job = sinfonia_cleanup_job,
 	.read_parse = kodak605_read_parse,
@@ -908,13 +971,6 @@ const struct dyesub_backend kodak605_backend = {
 	.query_markers = kodak605_query_markers,
 	.query_serno = sinfonia_query_serno,
 	.query_stats = kodak605_query_stats,
-	.devices = {
-		{ USB_VID_KODAK, USB_PID_KODAK_605, P_KODAK_605, "Kodak", "kodak-605"},
-		{ USB_VID_KODAK, USB_PID_KODAK_7000, P_KODAK_7000, "Kodak", "kodak-7000"},
-		{ USB_VID_KODAK, USB_PID_KODAK_7010, P_KODAK_701X, "Kodak", "kodak-7010"},
-		{ USB_VID_KODAK, USB_PID_KODAK_7015, P_KODAK_701X, "Kodak", "kodak-7015"}, /* Duplicate */
-		{ 0, 0, 0, NULL, NULL}
-	}
 };
 
 /* Kodak 605/70xx data format
@@ -932,16 +988,20 @@ const struct dyesub_backend kodak605_backend = {
   CC CC                          Number of copies (1-???)
   WW WW                          Number of columns (Fixed at 1844 or 1548)
   HH HH                          Number of rows
-  DD                             0x01 (4x6) 0x03 (8x6)
+  DD                             Print Mode/Media? See below
   LL                             Laminate, 0x01/0x02/0x03 (off/on/satin[70xx only])
-  00                             Print Mode (???)
+  00                             Print Method?
 
-  ************************************************************************
+   Media/Mode codes:
 
-  Note:  Kodak 605  is actually a Shinko CHC-S1545-5A
-  Note:  Kodak 7000 is actually a Shinko CHC-S1645-5A
-  Note:  Kodak 7010 is actually a Shinko CHC-S1645-5B
-  Note:  Kodak 7015 is actually a Shinko CHC-S1645-5C
+     0x01   6x4
+     0x03   6x8
+     0x06   5x7
+     0x07   5x4
+     0x08   5x5
+     0x09   5x7.5
+     0x0d   5x3.5
+     0x0e   6x6
 
   ************************************************************************
 
