@@ -1,13 +1,13 @@
 /*
- *   Shinko/Sinfonia CHC-S6245 CUPS backend -- libusb-1.0 version
+ *   Shinko/Sinfonia CHC-S6245 CUPS backend
  *
- *   (c) 2015-2020 Solomon Peachy <pizza@shaftnet.org>
+ *   (c) 2015-2024 Solomon Peachy <pizza@shaftnet.org>
  *
  *   Low-level documentation was provided by Sinfonia, Inc.  Thank you!
  *
  *   The latest version of this program can be found at:
  *
- *     http://git.shaftnet.org/cgit/selphy_print.git
+ *     https://git.shaftnet.org/gitea/slp/selphy_print.git
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the Free
@@ -20,9 +20,7 @@
  *   for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
- *
- *          [http://www.gnu.org/licenses/gpl-2.0.html]
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  *   SPDX-License-Identifier: GPL-2.0+
  *
@@ -59,70 +57,66 @@ struct s6245_print_cmd {
 
 static const struct sinfonia_param ek8810_params[] =
 {
-	{ 0x01, "Unknown_01" }, // 00000001
-	{ 0x11, "Unknown_11" }, // 00000001
-	{ 0x12, "? Matte Gloss" }, // 00000069
-	{ 0x13, "? Matte Degloss Black" }, // 000000c3
-	{ 0x14, "? Matte Degloss White" }, // 000000cd
-	{ 0x21, "Unknown_21" }, // 000003e8
-	{ 0x22, "Unknown_22" }, // 0000041a
-	{ 0x23, "Unknown_23" }, // 00000152
-	{ 0x24, "Unknown_24" }, // 0000044c
-	{ 0x25, "Unknown_25" }, // 0000044c
+	{ PARAM_UNK_01, "Unknown_01" },
+	{ PARAM_UNK_11, "Unknown_11" },
+	{ PARAM_UNK_12, "? Matte Gloss" },
+	{ PARAM_UNK_13, "? Matte Degloss Black" },
+	{ PARAM_UNK_14, "? Matte Degloss White" },
+	{ PARAM_UNK_21, "Unknown_21" },
+	{ PARAM_UNK_22, "Unknown_22" },
+	{ PARAM_UNK_23, "Unknown_23" },
+	{ PARAM_UNK_24, "Unknown_24" },
+	{ PARAM_UNK_25, "Unknown_25" },
 
-	{ 0x2f, "Unknown_2f" }, // 00000320
-	{ 0x41, "Unknown_41" }, // 0000005d
-	{ 0x42, "Unknown_42" }, // 00000048
-	{ 0x43, "Unknown_43" }, // 0000007c
-	{ 0x44, "Unknown_44" }, // 00000088
-	{ 0x45, "Unknown_45" }, // 00000000
-	{ 0x46, "Unknown_46" }, // 00000002
-	{ 0x47, "Unknown_47" }, // 00000063
-	{ 0x48, "Unknown_48" }, // 00000008
-	{ 0x61, "Unknown_61" }, // 00000050
+	{ PARAM_UNK_2F, "Unknown_2f" },
+	{ PARAM_UNK_41, "Unknown_41" },
+	{ PARAM_UNK_42, "Unknown_42" },
+	{ PARAM_UNK_43, "Unknown_43" },
+	{ PARAM_UNK_44, "Unknown_44" },
+	{ PARAM_UNK_45, "Unknown_45" },
+	{ PARAM_UNK_46, "Unknown_46" },
+	{ PARAM_UNK_47, "Unknown_47" },
+	{ PARAM_UNK_48, "Unknown_48" },
+	{ PARAM_UNK_61, "Unknown_61" },
 
-	{ 0x62, "Unknown_62" }, // 00000031
-	{ 0x63, "Unknown_63" }, // 00000030
-	{ 0x64, "Unknown_64" }, // 00000030
-	{ 0x81, "Unknown_81" }, // ffffffff
-	{ 0x82, "Unknown_82" }, // fffffff9
-	{ 0x83, "Unknown_83" }, // fffffffc
-	{ 0x84, "Unknown_84" }, // 00000002
-	{ 0x8a, "Unknown_8a" }, // 00000005
-	{ 0x8b, "Unknown_8b" }, // 00000005
-	{ 0x8c, "Unknown_8c" }, // 00000000
+	{ PARAM_UNK_62, "Unknown_62" },
+	{ PARAM_UNK_63, "Unknown_63" },
+	{ PARAM_UNK_64, "Unknown_64" },
+	{ PARAM_UNK_81, "Unknown_81" },
+	{ PARAM_UNK_82, "Unknown_82" },
+	{ PARAM_UNK_83, "Unknown_83" },
+	{ PARAM_UNK_84, "Unknown_84" },
+	{ PARAM_UNK_8A, "Unknown_8a" },
+	{ PARAM_UNK_8B, "Unknown_8b" },
+	{ PARAM_UNK_8C, "Unknown_8c" },
 
-	{ 0x8d, "Unknown_8d" }, // 00000000
-	{ 0x91, "Unknown_91" }, // 0000007e
-	{ 0x92, "Unknown_92" }, // 0000007d
-	{ 0x93, "Unknown_93" }, // 00000077
-	{ 0xa0, "Unknown_a0" }, // 00000005
-	{ 0xa1, "Unknown_a1" }, // 00000000
-	{ 0xa2, "Unknown_a2" }, // 00000008
-	{ 0xa3, "Unknown_a3" }, // 00000030
-	{ 0xa4, "Unknown_a4" }, // 00000030
-	{ 0xa5, "? Thermal Protect Lamination" }, // 00000046
+	{ PARAM_UNK_8D, "Unknown_8d" },
+	{ PARAM_UNK_91, "Unknown_91" },
+	{ PARAM_UNK_92, "Unknown_92" },
+	{ PARAM_UNK_93, "Unknown_93" },
+	{ PARAM_UNK_A0, "Unknown_a0" },
+	{ PARAM_UNK_A1, "Unknown_a1" },
+	{ PARAM_UNK_A2, "Unknown_a2" },
+	{ PARAM_UNK_A3, "Unknown_a3" },
+	{ PARAM_UNK_A4, "Unknown_a4" },
+	{ PARAM_UNK_A5, "? Thermal Protect Lamination" },
 
-	{ 0xa6, "Unknown_a6" }, // 00000001
-	{ 0xa7, "Unknown_a7" }, // 00000014
-	{ 0xa8, "Unknown_a8" }, // 00000001
-	{ 0xa9, "Unknown_a9" }, // ffffffff
-	{ 0xc1, "Unknown_c1" }, // 00000002
-	{ 0xc2, "Unknown_c2" }, // 000000c8
-	{ 0xc3, "Unknown_c3" }, // 000000c8
-	{ 0xc4, "Unknown_c4" }, // 000004d0
-	{ 0xf1, "Unknown_f1" }, // 00000022
-	{ 0xf2, "Unknown_f2" }, // 00000022
+	{ PARAM_UNK_A6, "Unknown_a6" },
+	{ PARAM_UNK_A7, "Unknown_a7" },
+	{ PARAM_UNK_A8, "Unknown_a8" },
+	{ PARAM_UNK_A9, "Unknown_a9" },
+	{ PARAM_UNK_C1, "Unknown_c1" },
+	{ PARAM_UNK_C2, "Unknown_c2" },
+	{ PARAM_UNK_C3, "Unknown_c3" },
+	{ PARAM_UNK_C4, "Unknown_c4" },
+	{ PARAM_UNK_F1, "Unknown_f1" },
+	{ PARAM_UNK_F2, "Unknown_f2" },
 
-	{ 0xf3, "Unknown_f3" }, // 00000047
-	{ 0xf4, "Unknown_f4" }, // 00000022
+	{ PARAM_UNK_F3, "Unknown_f3" },
+	{ PARAM_UNK_F4, "Unknown_f4" },
 };
 #define ek8810_params_num (sizeof(ek8810_params) / sizeof(struct sinfonia_param))
 
-#define PARAM_DRIVER_MODE  0x3e
-#define PARAM_PAPER_MODE   0x3f
-#define PARAM_SLEEP_TIME   0x54
-
 static const struct sinfonia_param s6245_params[] =
 {
 	{ PARAM_DRIVER_MODE, "Driver Mode/Wizard" },
@@ -131,19 +125,6 @@ static const struct sinfonia_param s6245_params[] =
 };
 #define s6245_params_num (sizeof(s6245_params) / sizeof(struct sinfonia_param))
 
-#define PARAM_DRIVER_WIZOFF 0x00000000
-#define PARAM_DRIVER_WIZON  0x00000001
-
-#define PARAM_PAPER_NOCUT   0x00000000
-#define PARAM_PAPER_CUTLOAD 0x00000001
-
-#define PARAM_SLEEP_5MIN    0x00000000
-#define PARAM_SLEEP_15MIN   0x00000001
-#define PARAM_SLEEP_30MIN   0x00000002
-#define PARAM_SLEEP_60MIN   0x00000003
-#define PARAM_SLEEP_120MIN  0x00000004
-#define PARAM_SLEEP_240MIN  0x00000005
-
 static const char *s6245_error_codes(uint8_t major, uint8_t minor)
 {
 	switch(major) {
@@ -816,7 +797,7 @@ static void shinkos6245_cmdline(void)
 	DEBUG("\t\t[ -R ]           # Reset printer to factory defaults\n");
 	DEBUG("\t\t[ -s ]           # Query status\n");
 	DEBUG("\t\t[ -X jobid ]     # Abort a printjob\n");
-//	DEBUG("\t\t[ -Z ]           # Dump all parameters\n");
+//	DEBUG("\t\t[ -Z 0 | 1 ]     # Dump all parameters\n");
 }
 
 static int shinkos6245_cmdline_arg(void *vctx, int argc, char **argv)
@@ -827,7 +808,7 @@ static int shinkos6245_cmdline_arg(void *vctx, int argc, char **argv)
 	if (!ctx)
 		return -1;
 
-	while ((i = getopt(argc, argv, GETOPT_LIST_GLOBAL "b:c:C:eFik:l:L:mrR:sX:Z:")) >= 0) {
+	while ((i = getopt(argc, argv, GETOPT_LIST_GLOBAL "b:c:C:eFik:l:L:mrR:sX:Z")) >= 0) {
 		switch(i) {
 		GETOPT_PROCESS_GLOBAL
 		case 'b':
@@ -955,7 +936,7 @@ static int shinkos6245_attach(void *vctx, struct dyesub_connection *conn, uint8_
 	} else {
 		int media_code = RIBBON_8x12;
 		if (getenv("MEDIA_CODE"))
-			media_code = atoi(getenv("MEDIA_CODE"));
+			media_code = strtol(getenv("MEDIA_CODE"),NULL,16);
 
 		ctx->media.ribbon_code = media_code;
 	}
@@ -995,11 +976,40 @@ static int shinkos6245_read_parse(void *vctx, const void **vjob, int data_fd, in
 		return ret;
 	}
 
-	/* Use whicever copy count is larger */
-	if ((int)job->jp.copies > copies)
-		job->copies = job->jp.copies;
-	else
-		job->copies = copies;
+	/* Use larger of our copy counts */
+	if (job->common.copies < copies)
+		job->common.copies = copies;
+
+	if (ctx->dev.conn->type == P_KODAK_8810 &&
+	    job->jp.rows > 3624) {
+		if (job->common.copies > 1) {
+			WARNING("Multiple copies of panorama prints is not supported!\n");
+			job->common.copies = 1;
+		}
+
+		if (job->jp.media) {
+			ERROR("Don't support multi-cut with panorama!\n");
+			sinfonia_cleanup_job(job);
+			return CUPS_BACKEND_CANCEL;
+		}
+
+		int rval;
+		int maxrows;
+
+		if (ctx->media.ribbon_code != RIBBON_8x12 &&
+		    ctx->media.ribbon_code != RIBBON_8x12K)
+			maxrows = 3024;
+		else
+			maxrows = 3624;
+
+		rval = sinfonia_panorama_splitjob(job, maxrows,
+						   (struct sinfonia_printjob**)vjob);
+
+		/* Unconditionally clean up original job regardless */
+		sinfonia_cleanup_job(job);
+
+		return rval;
+	}
 
 	*vjob = job;
 
@@ -1062,7 +1072,7 @@ static struct kodak8810_cutlist cutlist_8x4x3_d = {
 	.cut[5] = 3624,
 };
 
-static int shinkos6245_main_loop(void *vctx, const void *vjob) {
+static int shinkos6245_main_loop(void *vctx, const void *vjob, int wait_for_return) {
 	struct shinkos6245_ctx *ctx = vctx;
 
 	int ret, num;
@@ -1079,13 +1089,16 @@ static int shinkos6245_main_loop(void *vctx, const void *vjob) {
 	struct sinfonia_printjob *job = (struct sinfonia_printjob*) vjob;
 	struct kodak8810_cutlist *cutlist = NULL;
 
-	copies = job->copies;
+	copies = job->common.copies;
 
 	/* Cap copies */
-	// XXX 120 for 8x10 media, 100 for 8x12 media (S6245 / P910L)
-	// 250 for 8x12, 300 for 8x10 (Kodak 8810)
-	if (copies > 120)
-		copies = 120;
+	if (ctx->dev.conn->type == P_KODAK_8810) {
+		if (copies > 250)
+			copies = 250; // XXX 300 for 8x12 media
+	} else {
+		if (copies > 120)
+			copies = 120; // XXX 100 for 8x12 media
+	}
 
 	/* Set up mcut */
 	switch (job->jp.media) {
@@ -1103,8 +1116,8 @@ static int shinkos6245_main_loop(void *vctx, const void *vjob) {
 	}
 	// XXX what about mcut |= PRINT_METHOD_DISABLE_ERR;
 
-	/* EK8810 uses special "cutlist" */
 	if (ctx->dev.conn->type == P_KODAK_8810) {
+		/* EK8810 uses special "cutlist" */
 		switch (job->jp.media) {
 		case CODE_8x4_2:
 			if (job->jp.ext_flags & EXT_FLAG_DOUBLESLUG)
@@ -1135,34 +1148,32 @@ static int shinkos6245_main_loop(void *vctx, const void *vjob) {
 		}
 	}
 
-#if 0  /* Doesn't work on EK8810.  Not sure about S6245 */
-	int i;
-	/* Validate print sizes */
-	for (i = 0; i < ctx->media.count ; i++) {
-		/* Look for matching media */
-		if (ctx->media.items[i].columns == job->jp.columns &&
-		    ctx->media.items[i].rows == job->jp.rows)
-			break;
+	if (ctx->dev.conn->type == P_KODAK_8810) {
+		if (ctx->media.ribbon_code != RIBBON_8x12K &&
+		    job->jp.rows > 3024) { // 3036 on 6245
+			ERROR("Incorrect media loaded for print!\n");
+			return CUPS_BACKEND_HOLD;
+		}
+	} else {
+		int i;
+		/* Validate print sizes */
+		for (i = 0; i < ctx->media.count ; i++) {
+			/* Look for matching media */
+			if (ctx->media.items[i].columns == job->jp.columns &&
+			    ctx->media.items[i].rows == job->jp.rows)
+				break;
+		}
+		if (i == ctx->media.count) {
+			ERROR("Incorrect media loaded for print!\n");
+			return CUPS_BACKEND_HOLD;
+		}
 	}
-	if (i == ctx->media.count) {
-		ERROR("Incorrect media loaded for print!\n");
-		return CUPS_BACKEND_HOLD;
-	}
-#else
-	if (ctx->media.ribbon_code != RIBBON_8x12 &&
-	    ctx->media.ribbon_code != RIBBON_8x12K &&
-	    job->jp.rows > 3024) {
-		ERROR("Incorrect media loaded for print!\n");
-		return CUPS_BACKEND_HOLD;
-	}
-
-#endif
 
 	/* Send Set Time */
 	if (ctx->dev.conn->type != P_KODAK_8810) {
 		struct sinfonia_settime_cmd *settime = (struct sinfonia_settime_cmd *)cmdbuf;
 		time_t now = time(NULL);
-		struct tm *cur = localtime(&now);
+		const struct tm *cur = localtime(&now);
 
 		memset(cmdbuf, 0, CMDBUF_LEN);
 		cmd->cmd = cpu_to_le16(SINFONIA_CMD_SETTIME);
@@ -1309,7 +1320,7 @@ top:
 		state = S_PRINTER_SENT_DATA;
 		break;
 	case S_PRINTER_SENT_DATA:
-		if (fast_return) {
+		if (!wait_for_return) {
 			INFO("Fast return mode enabled.\n");
 			state = S_FINISHED;
 		} else if (sts.hdr.status == STATUS_READY) {
@@ -1447,14 +1458,6 @@ static int shinkos6245_query_stats(void *vctx,  struct printerstats *stats)
 	return CUPS_BACKEND_OK;
 }
 
-/* Exported */
-#define USB_VID_SHINKO       0x10CE
-#define USB_PID_SHINKO_S6245 0x001D
-#define USB_VID_HITI         0x0D16
-#define USB_PID_HITI_P910L   0x000E
-#define USB_VID_KODAK        0x040A
-#define USB_PID_KODAK_8810   0x404D
-
 static const char *shinkos6245_prefixes[] = {
 	"shinkos6245", /* Family Name */
 	// backwards compatibility
@@ -1462,9 +1465,17 @@ static const char *shinkos6245_prefixes[] = {
 	NULL
 };
 
+static const struct device_id shinkos6245_devices[] = {
+	{ 0x10ce, 0x001d, P_SHINKO_S6245, NULL, "sinfonia-chcs6245"},
+	{ 0x10ce, 0x001d, P_SHINKO_S6245, NULL, "shinko-chcs6245"}, /* Duplicate */
+	{ 0x0d16, 0x000e, P_HITI_910, NULL, "hiti-p910l"}, /* aka CHC-S6245-5B */
+	{ 0x040a, 0x404d, P_KODAK_8810, NULL, "kodak-8810"}, /* aka CHC-S1845-5A */
+	{ 0, 0, 0, NULL, NULL}
+};
+
 const struct dyesub_backend shinkos6245_backend = {
 	.name = "Sinfonia CHC-S6245 / Kodak 8810",
-	.version = "0.36" " (lib " LIBSINFONIA_VER ")",
+	.version = "0.43" " (lib " LIBSINFONIA_VER ")",
 	.uri_prefixes = shinkos6245_prefixes,
 	.cmdline_usage = shinkos6245_cmdline,
 	.cmdline_arg = shinkos6245_cmdline_arg,
@@ -1476,13 +1487,7 @@ const struct dyesub_backend shinkos6245_backend = {
 	.query_serno = sinfonia_query_serno,
 	.query_markers = shinkos6245_query_markers,
 	.query_stats = shinkos6245_query_stats,
-	.devices = {
-		{ USB_VID_SHINKO, USB_PID_SHINKO_S6245, P_SHINKO_S6245, NULL, "sinfonia-chcs6245"},
-		{ USB_VID_SHINKO, USB_PID_SHINKO_S6245, P_SHINKO_S6245, NULL, "shinko-chcs6245"}, /* Duplicate */
-		{ USB_VID_HITI, USB_PID_HITI_P910L, P_HITI_910, NULL, "hiti-p910l"},
-		{ USB_VID_KODAK, USB_PID_KODAK_8810, P_KODAK_8810, NULL, "kodak-8810"},
-		{ 0, 0, 0, NULL, NULL}
-	}
+	.devices = shinkos6245_devices,
 };
 
 /* CHC-S6245 data format
