@@ -1,11 +1,11 @@
  /*
  *   Shinko/Sinfonia Common Code
  *
- *   (c) 2019-2020 Solomon Peachy <pizza@shaftnet.org>
+ *   (c) 2019-2025 Solomon Peachy <pizza@shaftnet.org>
  *
  *   The latest version of this program can be found at:
  *
- *     http://git.shaftnet.org/cgit/selphy_print.git
+ *     https://git.shaftnet.org/gitea/slp/selphy_print.git
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the Free
@@ -34,6 +34,8 @@ int sinfonia_read_parse(int data_fd, uint32_t model,
 	int ret, i;
 	uint8_t tmpbuf[4];
 
+	job->common.jobsize = sizeof(*job);
+
 	/* Read in header */
 	ret = read(data_fd, hdr, SINFONIA_HDR_LEN);
 	if (ret < 0 || ret != SINFONIA_HDR_LEN) {
@@ -129,7 +131,7 @@ int sinfonia_read_parse(int data_fd, uint32_t model,
 	}
 	job->jp.columns = hdr[13];
 	job->jp.rows = hdr[14];
-	job->jp.copies = hdr[15];
+	job->common.copies = hdr[15];
 
 	if (hdr[1] == 2245 || hdr[1] == 6145)
 		job->jp.ext_flags = hdr[28];
@@ -142,6 +144,8 @@ int sinfonia_raw10_read_parse(int data_fd, struct sinfonia_printjob *job)
 	struct sinfonia_printcmd10_hdr hdr;
 	int ret;
 
+	job->common.jobsize = sizeof(*job);
+
 	/* Read in header */
 	ret = read(data_fd, &hdr, sizeof(hdr));
 	if (ret < 0 || ret != sizeof(hdr)) {
@@ -158,7 +162,7 @@ int sinfonia_raw10_read_parse(int data_fd, struct sinfonia_printjob *job)
 		ERROR("Unrecognized data format!\n");
 		return CUPS_BACKEND_CANCEL;
 	}
-	job->jp.copies = le16_to_cpu(hdr.copies);
+	job->common.copies = le16_to_cpu(hdr.copies);
 	job->jp.rows = le16_to_cpu(hdr.rows);
 	job->jp.columns = le16_to_cpu(hdr.columns);
 	job->jp.media = hdr.media;
@@ -169,8 +173,8 @@ int sinfonia_raw10_read_parse(int data_fd, struct sinfonia_printjob *job)
 	job->datalen = job->jp.rows * job->jp.columns * 3;
 
 	/* Hack in backprinting */
-	if (job->jp.copies & 0x8000) {
-		job->jp.copies &= ~0x8000;
+	if (job->common.copies & 0x8000) {
+		job->common.copies &= ~0x8000;
 		job->datalen += (44 * 2);
 		job->jp.ext_flags = EXT_FLAG_BACKPRINT;
 	}
@@ -200,11 +204,148 @@ int sinfonia_raw10_read_parse(int data_fd, struct sinfonia_printjob *job)
 	return CUPS_BACKEND_OK;
 }
 
+int sinfonia_panorama_splitjob(struct sinfonia_printjob *injob,
+			       uint16_t max_rows,
+			       struct sinfonia_printjob **newjobs)
+{
+	uint8_t *panels[3] = { NULL, NULL, NULL };
+	uint16_t panel_rows[3] = { 0, 0, 0 };
+	uint8_t numpanels;
+	uint16_t overlap_rows;
+	uint16_t pad_rows;
+	uint16_t inrows;
+	uint16_t cols;
+
+	inrows = injob->jp.rows;
+	cols = injob->jp.columns;
+
+	switch (cols) {
+	case 1548: /* EK6900/6950 */
+		if (max_rows != 2136) {
+			ERROR("Bad pano input\n");
+			return CUPS_BACKEND_CANCEL;
+		}
+		if (inrows > 3036) // 5x10
+			numpanels = 3;
+		else
+			numpanels = 2;
+
+		pad_rows = 18;
+		overlap_rows = 600;
+
+		if (numpanels == 3) {
+			if (inrows > 4536) // 5x15
+				overlap_rows = 100;
+			else
+				overlap_rows = 600;
+		}
+		break;
+	case 1844: /* EK6900/6950 */
+		if (max_rows != 2436) {
+			ERROR("Bad pano input\n");
+			return CUPS_BACKEND_CANCEL;
+		}
+
+		pad_rows = 18;
+		overlap_rows = 600;
+
+		if (inrows > 4236) // ie 6x14
+			numpanels = 3;
+		else
+			numpanels = 2;
+
+		break;
+	case 2464: /* EK8810 */
+		if (max_rows != 3624 && max_rows != 3024) {
+			ERROR("Bad pano input\n");
+			return CUPS_BACKEND_CANCEL;
+		}
+		pad_rows = 12;
+		overlap_rows = 600;
+
+		if (max_rows == 3024) { /* 8x10 media */
+			if (inrows > 5424) // 8x18
+				numpanels = 3;
+			else
+				numpanels = 2;
+		} else { /* 8x12 media */
+			if (inrows > 6624) // 8x22
+				numpanels = 3;
+			else
+				numpanels = 2;
+		}
+
+		if (numpanels == 3) {
+			if (max_rows == 3024) {
+				if (inrows == 6024) // 8x20
+					overlap_rows = 0;
+			} else {
+				if (inrows == 10824) // 8x36
+					overlap_rows = 0;
+			}
+		} else {
+			if (max_rows == 3024) {
+				if (inrows == 9024) // 8x30
+					overlap_rows = 0;
+			} else {
+				if (inrows == 7224) // 8x24
+					overlap_rows = 0;
+			}
+		}
+
+		break;
+	default:
+		ERROR("Unknown pano input cols: %d\n", cols);
+		return CUPS_BACKEND_CANCEL;
+	}
+
+	/* Work out panel sizes... */
+	for (int src_rows = inrows - 2 * pad_rows, i = 0 ; src_rows > 0; ) {
+		panel_rows[i] = (src_rows < max_rows) ? src_rows : max_rows - 2*pad_rows;
+		src_rows -= panel_rows[i];
+		i++;
+
+		if (i < numpanels)
+			src_rows += overlap_rows;
+	}
+
+	/* Allocate and set up new jobs and buffers */
+	for (int i = 0 ; i < numpanels ; i++) {
+		newjobs[i] = malloc(sizeof(struct sinfonia_printjob));
+		if (!newjobs[i]) {
+			ERROR("Memory allocation failure");
+			return CUPS_BACKEND_RETRY_CURRENT;
+		}
+		panel_rows[i] += 2 * pad_rows;
+		panels[i] = malloc(cols * panel_rows[i] * 3);
+		if (!panels[i]) {
+			ERROR("Memory allocation failure");
+			return CUPS_BACKEND_RETRY_CURRENT;
+		}
+		/* Fill in header differences */
+		memcpy(newjobs[i], injob, sizeof(struct sinfonia_printjob));
+		newjobs[i]->databuf = panels[i];
+		newjobs[i]->datalen = panel_rows[i] * cols;
+		newjobs[i]->jp.rows = panel_rows[i];
+		// XXX what else?
+
+		panel_rows[i] -= 2 * pad_rows;
+	}
+
+	dyesub_pano_split_rgb8(injob->databuf, cols, numpanels,
+			       overlap_rows, pad_rows,
+			       panel_rows, panels);
+
+	return CUPS_BACKEND_OK;
+}
+
 int sinfonia_raw18_read_parse(int data_fd, struct sinfonia_printjob *job)
 {
 	struct sinfonia_printcmd18_hdr hdr;
 	int ret;
 
+	job->common.jobsize = sizeof(*job);
+
 	/* Read in header */
 	ret = read(data_fd, &hdr, sizeof(hdr));
 	if (ret < 0 || ret != sizeof(hdr)) {
@@ -221,7 +362,7 @@ int sinfonia_raw18_read_parse(int data_fd, struct sinfonia_printjob *job)
 		ERROR("Unrecognized data format!\n");
 		return CUPS_BACKEND_CANCEL;
 	}
-	job->jp.copies = le16_to_cpu(hdr.copies);
+	job->common.copies = le16_to_cpu(hdr.copies);
 	job->jp.rows = le16_to_cpu(hdr.rows);
 	job->jp.columns = le16_to_cpu(hdr.columns);
 	job->jp.media = hdr.media;
@@ -260,6 +401,8 @@ int sinfonia_raw28_read_parse(int data_fd, struct sinfonia_printjob *job)
 	struct sinfonia_printcmd28_hdr hdr;
 	int ret;
 
+	job->common.jobsize = sizeof(*job);
+
 	/* Read in header */
 	ret = read(data_fd, &hdr, sizeof(hdr));
 	if (ret < 0 || ret != sizeof(hdr)) {
@@ -276,7 +419,7 @@ int sinfonia_raw28_read_parse(int data_fd, struct sinfonia_printjob *job)
 		ERROR("Unrecognized data format!\n");
 		return CUPS_BACKEND_CANCEL;
 	}
-	job->jp.copies = le16_to_cpu(hdr.copies);
+	job->common.copies = le16_to_cpu(hdr.copies);
 	job->jp.rows = le16_to_cpu(hdr.rows);
 	job->jp.columns = le16_to_cpu(hdr.columns);
 	job->jp.media = hdr.media;
@@ -322,13 +465,13 @@ void sinfonia_cleanup_job(const void *vjob)
 }
 
 int sinfonia_docmd(struct sinfonia_usbdev *usbh,
-		   uint8_t *cmd, int cmdlen,
+		   const uint8_t *cmd, int cmdlen,
 		   uint8_t *resp, int resplen,
 		   int *num)
 {
 	int ret;
 
-	struct sinfonia_cmd_hdr *cmdhdr =  (struct sinfonia_cmd_hdr *) cmd;
+	const struct sinfonia_cmd_hdr *cmdhdr =  (struct sinfonia_cmd_hdr *) cmd;
 	struct sinfonia_status_hdr *resphdr = (struct sinfonia_status_hdr *)resp;
 
 	if ((ret = send_data(usbh->conn,
@@ -348,14 +491,13 @@ int sinfonia_docmd(struct sinfonia_usbdev *usbh,
 		INFO(" Result: 0x%02x  Error: 0x%02x (0x%02x/0x%02x = %s)\n",
 		     resphdr->result, resphdr->error, resphdr->printer_major,
 		     resphdr->printer_minor, usbh->error_codes(resphdr->printer_major, resphdr->printer_minor));
-		ret = CUPS_BACKEND_FAILED;
 		goto fail;
 	}
 
 	return CUPS_BACKEND_OK;
 fail:
 	ERROR("Failed to execute %s command\n", sinfonia_cmd_names(cmdhdr->cmd));
-	return ret;
+	return CUPS_BACKEND_FAILED;
 }
 
 int sinfonia_flashled(struct sinfonia_usbdev *usbh)
@@ -478,6 +620,7 @@ int sinfonia_setparam(struct sinfonia_usbdev *usbh, int target, uint32_t param)
 				  &num))) {
 //		ERROR("Unable to query param id %02x: %s\n",
 //		      target, sinfonia_paramname(usbh, target));
+		return ret;
 	}
 
 	return CUPS_BACKEND_OK;
@@ -519,15 +662,17 @@ int sinfonia_getfwinfo(struct sinfonia_usbdev *usbh)
 		if (le16_to_cpu(resp.hdr.payload_len) != (sizeof(struct sinfonia_fwinfo_resp) - sizeof(struct sinfonia_status_hdr)))
 			continue;
 
-		INFO(" %s\t ver %02x.%02x\n", sinfonia_fwinfo_targets(i),
-		     resp.major, resp.minor);
-#if 0
-		INFO("  name:    '%s'\n", resp.name);
-		INFO("  type:    '%s'\n", resp.type);
-		INFO("  date:    '%s'\n", resp.date);
-		INFO("  version: %02x.%02x (CRC %04x)\n", resp.major, resp.minor,
-		     le16_to_cpu(resp.checksum));
-#endif
+		if (dyesub_debug) {
+			INFO("  id  :    '%s'\n", sinfonia_fwinfo_targets(i));
+			INFO("  name:    '%s'\n", resp.name);
+			INFO("  type:    '%s'\n", resp.type);
+			INFO("  date:    '%s'\n", resp.date);
+			INFO("  version: %02x.%02x (CRC %04x)\n", resp.major, resp.minor,
+			     le16_to_cpu(resp.checksum));
+		} else {
+			INFO(" %s\t ver %02x.%02x\n", sinfonia_fwinfo_targets(i),
+			     resp.major, resp.minor);
+		}
 	}
 	return CUPS_BACKEND_OK;
 }
@@ -677,7 +822,7 @@ int sinfonia_button_set(struct sinfonia_usbdev *dev, int enable)
 
 	if ((ret = sinfonia_docmd(dev,
 				(uint8_t*)&cmd, sizeof(cmd),
-				(uint8_t*)&cmd, sizeof(resp),
+				(uint8_t*)&resp, sizeof(resp),
 				&num))) {
 		return ret;
 	}
@@ -712,7 +857,7 @@ int sinfonia_settonecurve(struct sinfonia_usbdev *usbh, int target, char *fname)
 
 	/* Set up command */
 	cmd.target = target;
-	cmd.curve_id = TONE_CURVE_ID;
+	cmd.target2 = UPDATE_TARGET2_TONE;
 	cmd.reserved[0] = cmd.reserved[1] = cmd.reserved[2] = 0;
 	cmd.reset = 0;
 	cmd.size = cpu_to_le32(TONE_CURVE_SIZE * sizeof(uint16_t));
