@@ -101,6 +101,7 @@ const char *special_options[] =
   "OutputOrder",
   "Quality",
   "Duplex",
+  "Collate",
   NULL
 };
 
@@ -284,6 +285,12 @@ print_ppd_header(gpFile fp, ppd_type_t ppd_type, int model, const char *driver,
 		 const char *language, const stp_string_list_t *po,
 		 char **all_langs)
 {
+  (void)model;
+  (void)family;
+  (void)device_id;
+  (void)ppd_location;
+  (void)all_langs;
+
   char short_long_name[(PPD_MAX_SHORT_NICKNAME) + 1];
  /*
   * Write a standard header...
@@ -376,6 +383,15 @@ print_ppd_header_3(gpFile fp, ppd_type_t ppd_type, int model,
 		   const char *language, const stp_string_list_t *po,
 		   char **all_langs)
 {
+  (void)po;
+  (void)ppd_location;
+  (void)manufacturer;
+  (void)long_name;
+  (void)ppd_type;
+  (void)model;
+  (void)driver;
+  (void)family;
+
   int i;
   gpputs(fp, "*FileSystem:	False\n");
   gpputs(fp, "*LandscapeOrientation: Plus90\n");
@@ -384,8 +400,7 @@ print_ppd_header_3(gpFile fp, ppd_type_t ppd_type, int model,
   gpputs(fp, "*cupsVersion:	1.2\n");
 
   gpprintf(fp, "*cupsFilter:	\"application/vnd.cups-raster 100 rastertogutenprint.%s\"\n", GUTENPRINT_RELEASE_VERSION);
-  if (strcasecmp(manufacturer, "EPSON") == 0)
-    gpputs(fp, "*cupsFilter:	\"application/vnd.cups-command 33 commandtoepson\"\n");
+
   if (device_id)
     {
       if (strlen(device_id) > 200)
@@ -440,6 +455,13 @@ print_ppd_header_2(gpFile fp, ppd_type_t ppd_type, int model, const char *driver
 		   const char *language, const stp_string_list_t *po,
 		   char **all_langs)
 {
+  (void)po;
+  (void)all_langs;
+  (void)device_id;
+  (void)manufacturer;
+  (void)long_name;
+  (void)ppd_type;
+
   gpprintf(fp, "*StpDriverName:	\"%s\"\n", driver);
   gpprintf(fp, "*StpDriverModelFamily:	\"%d_%s\"\n", model, family);
   gpprintf(fp, "*StpPPDLocation: \"%s\"\n", ppd_location);
@@ -763,8 +785,10 @@ print_group_close(
     stp_parameter_class_t p_class,	/* I - Option class */
     stp_parameter_level_t p_level,	/* I - Option level */
     const char		 *language,	/* I - language */
-    const stp_string_list_t    *po)		/* I - Message catalog */
+    const stp_string_list_t    *po)	/* I - Message catalog */
 {
+  (void)po;
+  (void)language;
   print_group(fp, "Close", p_class, p_level, NULL, NULL);
 }
 
@@ -988,6 +1012,7 @@ print_one_option(gpFile fp, stp_vars_t *v, const stp_string_list_t *po,
       print_close_ui = 0;
       gpprintf(fp, "*CloseUI: *Stp%s\n\n", desc->name);
 
+#ifndef __APPLE__ /* This leads to outright _crashes_ on apple platforms */
       /*
        * Add custom option code and value parameter...
        */
@@ -996,7 +1021,7 @@ print_one_option(gpFile fp, stp_vars_t *v, const stp_string_list_t *po,
       gpprintf(fp, "*ParamCustomStp%s Value/%s: 1 points %d %d\n\n",
 	       desc->name, _("Value"), (int) desc->bounds.dimension.lower,
 	       (int) desc->bounds.dimension.upper);
-
+#endif
       break;
     case STP_PARAMETER_TYPE_INT:
       gpprintf(fp, "*OPOptionHints Stp%s: \"input spinbox\"\n", lparam->name);
@@ -1047,6 +1072,7 @@ print_one_localization(gpFile fp, const stp_string_list_t *po,
   int i;
   const stp_param_string_t *opt;
   char		dimstr[255];		/* Dimension string */
+  (void)lparam;
 
   gpprintf(fp, "*%s.Translation Stp%s/%s: \"\"\n", lang,
 	   desc->name, stp_i18n_lookup(po, desc->text));
@@ -1198,8 +1224,8 @@ write_ppd(
     const char		*filename,	/* I - input filename */
     int			compress)	/* I - compress output */
 {
-  int		i, j, k, l;		/* Looping vars */
-  int		num_opts;		/* Number of printer options */
+  unsigned int	i, j, k, l;		/* Looping vars */
+  unsigned int	num_opts;		/* Number of printer options */
   stp_resolution_t	xdpi, ydpi;	/* Resolution info */
   stp_vars_t	*v;			/* Variable info */
   const char	*driver;		/* Driver name */
@@ -1217,7 +1243,7 @@ write_ppd(
   int printer_is_color = 0;
   int simplified = ppd_type == PPD_SIMPLIFIED;
   int skip_color = ppd_type == PPD_NO_COLOR_OPTS;
-  int maximum_level = simplified ?
+  unsigned int maximum_level = simplified ?
     STP_PARAMETER_LEVEL_BASIC : STP_PARAMETER_LEVEL_ADVANCED4;
   char		*default_resolution = NULL;  /* Default resolution mapped name */
   stp_string_list_t *resolutions = stp_string_list_create();
@@ -1283,6 +1309,28 @@ write_ppd(
   else
     gpputs(fp, "*cupsManualCopies: True\n");
 
+  stp_describe_parameter(v, "CommandFilterName", &desc);
+  if (desc.p_type == STP_PARAMETER_TYPE_STRING_LIST && desc.is_active)
+    {
+      opt = stp_string_list_param(desc.bounds.str, 0);
+      gpprintf(fp, "*cupsFilter:	\"application/vnd.cups-command 33 %s\"\n", opt->name);
+    }
+  stp_parameter_description_destroy(&desc);
+
+  stp_describe_parameter(v, "CommandFilterCommands", &desc);
+  if (desc.p_type == STP_PARAMETER_TYPE_STRING_LIST && desc.is_active)
+    {
+      gpputs(fp, "*cupsCommands:   \"");
+      num_opts = stp_string_list_count(desc.bounds.str);
+      for (i = 0 ; i < num_opts; i++)
+        {
+           opt = stp_string_list_param(desc.bounds.str, i);
+           gpprintf(fp, "%s ", opt->name);
+        }
+      gpputs(fp, "\"\n");
+    }
+  stp_parameter_description_destroy(&desc);
+
   print_ppd_header_3(fp, ppd_type, model,
 		     driver, family, long_name,
 		     manufacturer, device_id, ppd_location, language, po,
@@ -1640,6 +1688,26 @@ write_ppd(
     }
   stp_parameter_description_destroy(&desc);
 
+  /* Collation */
+  stp_describe_parameter(v, "Collate", &desc);
+  if (desc.is_active)
+    {
+      gpprintf(fp, "*OpenUI *%s/%s: PickOne\n",
+               desc.name, stp_i18n_lookup(po, desc.text));
+      gpprintf(fp, "*OrderDependency: 10 AnySetup *%s\n", desc.name);
+      gpprintf(fp, "*OPOptionHints %s: \"checkbox\"\n", desc.name);
+      gpprintf(fp, "*Default%s: %s\n", desc.name,
+               desc.deflt.boolean ? "True" : "False");
+      gpprintf(fp, "*StpDefault%s: %s\n", desc.name,
+               desc.deflt.boolean ? "True" : "False");
+      gpprintf(fp, "*%s %s/%s: \"\"\n",
+               desc.name, "False", _("No"));
+      gpprintf(fp, "*%s %s/%s: \"\"\n",
+               desc.name, "True", _("Yes"));
+      gpprintf(fp, "*CloseUI: *%s\n\n", desc.name);
+    }
+  stp_parameter_description_destroy(&desc);
+
   gpprintf(fp, "*OpenUI *StpiShrinkOutput/%s: PickOne\n",
 	   _("Shrink Page If Necessary to Fit Borders"));
   gpputs(fp, "*OPOptionHints StpiShrinkOutput: \"radiobuttons\"\n");
@@ -1916,6 +1984,15 @@ write_ppd(
 	    }
 	  stp_parameter_description_destroy(&desc);
 
+          /* Collation */
+	  stp_describe_parameter(v, "Collate", &desc);
+	  if (desc.is_active)
+            {
+              gpprintf(fp, "*%s.Translation Collate/%s: \"\"\n", lang, _("Collate the Job"));
+              gpprintf(fp, "*%s.Collate %s/%s: \"\"\n", lang, "True", _("Yes"));
+              gpprintf(fp, "*%s.Collate %s/%s: \"\"\n", lang, "False", _("No"));
+            }
+
 	  gpprintf(fp, "*%s.Translation StpiShrinkOutput/%s: \"\"\n", lang,
 		   _("Shrink Page If Necessary to Fit Borders"));
 	  gpprintf(fp, "*%s.StpiShrinkOutput %s/%s: \"\"\n", lang, "Shrink", _("Shrink (print the whole page)"));
