@@ -1,4 +1,4 @@
-/* d4lib.c 
+/* d4lib.c
  * Copyright (C) 2001 Jean-Jacques Sarton jj.sarton@t-online.de
  *
  * This program is free software; you can redistribute it and/or modify
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 /* this file will be a library, which will allow to use the EPSON
@@ -31,10 +30,10 @@
  *
  * The best way to get the Stylus Scanner working is to test
  * this and also correct my possibly errors.
- * Programming knowledge will be helpfull for this.
+ * Programming knowledge will be helpful for this.
  *
  */
- 
+
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -47,6 +46,7 @@
 #include <errno.h>
 #include <string.h>
 #include <ctype.h>
+#include <time.h>
 
 #include "d4lib.h"
 
@@ -76,7 +76,7 @@ static int d4Errno = 0;
 /* commands for the D4 protocol
 
 Transaction    Cmd    Reply
--------------- ----   -----    
+-------------- ----   -----
 Init           0x00   0x80
 OpenChannel    0x01   0x81
 CloseChannel   0x02   0x82
@@ -145,28 +145,28 @@ typedef struct errorMessage_s
 
 static errorMessage_t errorMessage[] =
 {
-   { 0x01, "Unable to begin conversation, try later."              ,0 },
-   { 0x02, "Protocol revision not supported."                      ,1 },
-   { 0x03, "Transaction channel cant be closed."                  ,0 },
-   { 0x04, "No sufficient resources available now."                ,0 },
-   { 0x05, "Connection denied."                                    ,1 },
-   { 0x06, "Channel allready open."                                ,0 },
-   { 0x07, "Credit overflow, previous credit remain valid."        ,0 },
-   { 0x08, "Channel is not open."                                  ,1 },
-   { 0x09, "Service not available on specified socket."            ,1 },
-   { 0x0a, "Service name to socket ID failed."                     ,1 },
-   { 0x0b, "Init transaction failed."                              ,1 },
-   { 0x0c, "Invalid packet size."                                  ,1 },
-   { 0x0d, "Requested packed size is 0, no data can be transfered.",0 },
-   { 0x80, "Malformed packet, ignored."                            ,1 },
-   { 0x81, "No credit for received packet, ignored"                ,0 },
-   { 0x82, "Reply dont match with outstanding command, ignored."  ,1 },
-   { 0x83, "Packet size greater as negotiated size."               ,1 },
-   { 0x84, "Data received for a non opened channel."               ,1 },
-   { 0x85, "Reply with unknown result value received."             ,1 },
-   { 0x86, "Piggybacked credit in data packet cause overflow."     ,1 },
-   { 0x87, "Unknown 1284.4 Reply."                                 ,0 },
-   { 0x00, NULL                                                    ,0 }
+   { 0x01, "Unable to begin conversation, try later."                ,0 },
+   { 0x02, "Protocol revision not supported."                        ,1 },
+   { 0x03, "Transaction channel can't be closed."                    ,0 },
+   { 0x04, "No sufficient resources available now."                  ,0 },
+   { 0x05, "Connection denied."                                      ,1 },
+   { 0x06, "Channel already open."                                   ,0 },
+   { 0x07, "Credit overflow, previous credit remain valid."          ,0 },
+   { 0x08, "Channel is not open."                                    ,1 },
+   { 0x09, "Service not available on specified socket."              ,1 },
+   { 0x0a, "Service name to socket ID failed."                       ,1 },
+   { 0x0b, "Init transaction failed."                                ,1 },
+   { 0x0c, "Invalid packet size."                                    ,1 },
+   { 0x0d, "Requested packed size is 0, no data can be transferred." ,0 },
+   { 0x80, "Malformed packet, ignored."                              ,1 },
+   { 0x81, "No credit for received packet, ignored"                  ,0 },
+   { 0x82, "Reply doesn't match with outstanding command, ignored."  ,1 },
+   { 0x83, "Packet size greater as negotiated size."                 ,1 },
+   { 0x84, "Data received for a non opened channel."                 ,1 },
+   { 0x85, "Reply with unknown result value received."               ,1 },
+   { 0x86, "Piggybacked credit in data packet cause overflow."       ,1 },
+   { 0x87, "Unknown 1284.4 Reply."                                   ,0 },
+   { 0x00, NULL                                                      ,0 }
 };
 
 #define RESET_TIMER(ti,oti) { signal(SIGALRM, sig); \
@@ -183,6 +183,14 @@ static errorMessage_t errorMessage[] =
                                 sig = signal(SIGALRM, sigAlarm); \
                               }
 
+static int d4USleep(long usec)
+{
+  struct timespec t;
+  t.tv_sec = 0;
+  t.tv_nsec = 1000l * usec;
+  return nanosleep(&t, NULL);
+}
+
 /*******************************************************************/
 /* Function printHexValues                                         */
 /*                                                                 */
@@ -227,7 +235,7 @@ static void printHexValues(const char *dir, const unsigned char *buf, int len)
        for (i = 0; i < len; i++)
 	 {
 	   printf("%s%c", (print_prefix ? "+++" : ""),
-		  isprint(buf[i])||isblank(buf[i])?buf[i]:'*');
+		  isprint(buf[i])?buf[i]:'*');
 	   print_prefix = 0;
 	   if (buf[i] == ';' && i < len - 1)
 	     {
@@ -275,7 +283,7 @@ int SafeWrite(int fd, const void *data, int len)
     {
       status = write(fd, data, len);
       if(status < len)
-	usleep(d4WrTimeout);
+	d4USleep(d4WrTimeout);
       retries--;
     }
   while ((status < len) && (retries > 0));
@@ -292,6 +300,7 @@ int SafeWrite(int fd, const void *data, int len)
 
 static void sigAlarm(int code)
 {
+   (void)code;
    timeoutGot = -1;
 }
 
@@ -368,7 +377,7 @@ static void printCmdType(unsigned char *cmd)
 
 /*******************************************************************/
 /* Function writeCmd()                                             */
-/*        write a commmand                                         */
+/*        write a command                                          */
 /* Input:  int   fd    file handle                                 */
 /*         char *cmd   the data are to be put here                 */
 /*         int   len   the number of bytes to read                 */
@@ -405,13 +414,13 @@ static int writeCmd(int fd, unsigned char *cmd, int len)
 
    /* according to Glen Steward, this will solve problems  */
    /* for the cartridge exchange with the Stylus Color 580 */
-   usleep(d4MicroTimeout);
+   d4USleep(d4MicroTimeout);
 
    timeoutGot = 0;
    errno = 0;
    while ( i < len )
    {
-      SET_TIMER(ti,oti,d4WrTimeout);         
+      SET_TIMER(ti,oti,d4WrTimeout);
       w = SafeWrite(fd, cmd+i,len-i);
       RESET_TIMER(ti,oti);
       if ( w < 0 )
@@ -464,7 +473,7 @@ int readAnswer(int fd, unsigned char *buf, int len, int allowExtra)
    int first_read = 1;
    int excess = 0;
    /* wait a little bit before reading an answer */
-   usleep(d4RdTimeout);
+   d4USleep(d4RdTimeout);
 
    /* for error handling in case of timeout */
    timeoutGot = 0;
@@ -524,7 +533,7 @@ int readAnswer(int fd, unsigned char *buf, int len, int allowExtra)
          {
             /* the bytes idx 2 and 3 contain the length */
             /* in case of errors this may differ from   */
-            /* the expected lenght. Setting len to this */
+            /* the expected length. Setting len to this */
             /* value will avoid waiting for timeout     */
 	    int newlen = (buf[2] << 8) + buf[3];
 	    if (len > newlen)
@@ -542,7 +551,7 @@ int readAnswer(int fd, unsigned char *buf, int len, int allowExtra)
 	      }
          }
       }
-      usleep(d4RdTimeout);
+      d4USleep(d4RdTimeout);
    }
    if (debugD4)
      printf("\n");
@@ -563,13 +572,13 @@ int readAnswer(int fd, unsigned char *buf, int len, int allowExtra)
 	   else
 	     retry_count = 0;
 	   if (status < bytes)
-	     usleep(d4RdTimeout);
+	     d4USleep(d4RdTimeout);
 	   if (debugD4)
 	     printHexValues("waste", (const unsigned char *) wastebuf, status);
 	   excess -= status;
 	 }
      }
-   
+
    if ( debugD4 )
    {
       printCmdType(buf);
@@ -600,7 +609,7 @@ static void _flushData(int fd)
    char buf[1024];
    int len = 1023;
    int count = 200;
-   usleep(d4RdTimeout);
+   d4USleep(d4RdTimeout);
 
    /* for error handling in case of timeout */
    timeoutGot = 0;
@@ -613,7 +622,7 @@ static void _flushData(int fd)
      printf("+++flush data: length: %i\n", len);
    do
      {
-       usleep(d4RdTimeout);
+       d4USleep(d4RdTimeout);
        SET_TIMER(ti,oti, d4RdTimeout);
        rd = read(fd, buf, len);
        if (debugD4)
@@ -644,6 +653,7 @@ static int _readData(int fd, unsigned char *buf, int len)
    struct timeval beg, end;
    long dt;
    struct itimerval ti, oti;
+   (void)len;
 
    /* set errno to 0 in order to get correct informations */
    /* in case of error                                    */
@@ -681,7 +691,7 @@ static int _readData(int fd, unsigned char *buf, int len)
 
    if ( total == 6 )
    {
-      toGet = (header[2] >> 8) + header[3] - 6;
+      toGet = (header[2] << 8) + header[3] - 6;
       if (debugD4)
 	printf("+++toGet: %i\n", toGet);
       total = 0;
@@ -788,7 +798,7 @@ static int sendReceiveCmd(int fd, unsigned char *cmd, int len, unsigned char *an
 int EnterIEEE(int fd)
 {
    unsigned char buf[200];
-   unsigned char cmd[] = 
+   unsigned char cmd[] =
    {
       0x00, 0x00, 0x00, 0x1b, 0x01, '@', 'E', 'J', 'L', ' ',
       '1', '2', '8', '4', '.', '4', 0x0a, '@', 'E', 'J',
@@ -842,7 +852,7 @@ int Init(int fd)
    unsigned char buf[20];
    init_t cmd;
    int rd;
-   
+
    cmd.head.psid     = 0;
    cmd.head.ssid     = 0;
    cmd.head.lengthH  = 0;
@@ -851,7 +861,7 @@ int Init(int fd)
    cmd.head.control  = 0;
    cmd.head.command  = 0;
    cmd.revision      = 0x10;
-    
+
    rd = sendReceiveCmd(fd, (unsigned char*)&cmd, sizeof(cmd), buf, 9, 0 );
    return rd == 9 ? 1 : 0;
 }
@@ -1006,7 +1016,7 @@ int OpenChannel(int fd, unsigned char sockId, int *sndSz, int *rcvSz)
          return -1;
       }
    }
-   return 1;                 
+   return 1;
 }
 
 /*******************************************************************/
@@ -1076,7 +1086,7 @@ int CreditRequest(int fd, unsigned char socketID)
    else
    {
       return rd > 0 ? 0 : rd; /* there was an error */
-   } 
+   }
 }
 
 /*******************************************************************/
@@ -1140,11 +1150,11 @@ int askForCredit(int fd, unsigned char socketID, int *sndSize, int *rcvSize)
    int credit = 0;
    int count  = 0;
    int retries = 10;
-   
+
    while (credit == 0 && retries-- >= 0 )
    {
       while((credit=CreditRequest(fd,socketID)) == 0  && count < MAX_CREDIT_REQUEST && retries-- >= 0)
-         usleep(d4RdTimeout);
+         d4USleep(d4RdTimeout);
 
       if ( credit == -1 )
       {
@@ -1163,7 +1173,7 @@ int askForCredit(int fd, unsigned char socketID, int *sndSize, int *rcvSize)
             OpenChannel(fd, socketID, sndSize, rcvSize);
          }
       }
-      /* if the parent died, live this loop if credit not got */ 
+      /* if the parent died, live this loop if credit not got */
       if ( credit == 0 && getppid() == ppid )
          return 0;
       count++;
@@ -1240,7 +1250,7 @@ int writeData(int fd, unsigned char socketID, const unsigned char *buf, int len,
       gettimeofday(&end, NULL);
       dt = (end.tv_sec  - beg.tv_sec) * 1000000;
       dt += end.tv_usec - beg.tv_usec;
-# endif  
+# endif
       printf("+++Send: ");
       for ( ret = 0; (wr > 0) && (ret < ((wr > 20) ? 20 : wr)) ; ret++ )
          printf("%02x ", buffer[ret]);
@@ -1281,9 +1291,9 @@ int readData(int fd, unsigned char socketID, unsigned char *buf, int len)
    if ( Credit(fd, socketID, 1) == 1 )
    {
       /* wait a little bit */
-      usleep(d4RdDataTimeout);
+      d4USleep(d4RdDataTimeout);
       ret = _readData(fd, buf, len);
-      return ret; 
+      return ret;
    }
    else
    {
@@ -1304,7 +1314,7 @@ int readData(int fd, unsigned char socketID, unsigned char *buf, int len)
 /*         int   *sndSz    Send buffer size                        */
 /*         int   *rcvSz    Receive buffer size                     */
 /*         int   len       how many datas are to we send           */
-/*         fptr  test      function to verify buffer contents      */ 
+/*         fptr  test      function to verify buffer contents      */
 /*                                                                 */
 /* Return: number of bytes read or -1;                             */
 /*                                                                 */
@@ -1332,7 +1342,7 @@ int writeAndReadData(int fd, unsigned char socketID,
        /* wait a little bit */
        do
 	 {
-	   usleep(d4RdDataTimeout);
+	   d4USleep(d4RdDataTimeout);
 	   ret = _readData(fd, buf, len);
 	   if (ret < 0)
 	     return ret;
@@ -1366,7 +1376,7 @@ void flushData(int fd, unsigned char socketID)
        if ( Credit(fd, socketID, 1) == 1 )
 	 {
 	   /* wait a little bit */
-	   usleep(d4RdDataTimeout);
+	   d4USleep(d4RdDataTimeout);
 	   _flushData(fd);
 	 }
      }
@@ -1383,16 +1393,19 @@ void flushData(int fd, unsigned char socketID)
 /*                                                                 */
 /*******************************************************************/
 
-static void clearSndBuf(int fd)
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wunused-function"
+static inline void clearSndBuf(int fd)
 {
    char             buf[256];
    struct itimerval ti, oti;
-   
+
    SET_TIMER(ti,oti, d4RdTimeout);
    while ( read(fd, buf, sizeof(buf) ) > 0 )
       SET_TIMER(ti,oti, d4RdTimeout);
    RESET_TIMER(ti,oti);
 }
+#pragma GCC diagnostic pop
 
 void setDebug(int debug)
 {
@@ -1414,7 +1427,7 @@ int InitReply(int fd)
    cmd.head.command  = 0x80;
    cmd.head.result   = 0;   /* put the correct value here, 0,1,2 or 0x0b */
    cmd.revision      = 0x10;
-    
+
 }
 
 int ExitReply(int fd)
@@ -1443,7 +1456,7 @@ int Error(int fd)
    cmd.head.command  = 0x7f;
    cmd.epsid         = psid;
    cmd.essid         = ssid;
-   cmd.ecode         = 0x80; /* + 1...7 */ 
+   cmd.ecode         = 0x80; /* + 1...7 */
 }
 
 int GetSocketIDReply(int fd, unsigned char socketID)
@@ -1462,7 +1475,7 @@ int GetSocketIDReply(int fd, unsigned char socketID)
    cmd->result   = 0; /* put the correct vale here 0 or 0x0a */
    buf[sizeof(cmdHeader_t)] = socketID;
    strcpy(buf+1+sizeof(cmdHeader_t), serviceName);
-   
+
 }
 
 int GetServiceName(inf fd, unsigned char socketID)
@@ -1497,8 +1510,8 @@ int GetServiceNameReply(inf fd)
    cmd->result   = 0; /* put the correct vale here 0 or 0x0a */
    buf[sizeof(replyHeader_t)] = socketID;
    strcpy(buf+1+sizeof(replyHeader_t), serviceName);
-   
-}  
+
+}
 
 int CreditReply(int fd, unsigned char socketID, int credit)
 {
@@ -1516,4 +1529,3 @@ int CreditReply(int fd, unsigned char socketID, int credit)
 }
 
 #endif
-
