@@ -114,8 +114,15 @@ stpui_get_thumbnail_data_function(void *image_ID, gint *width, gint *height,
 {
   if (gimp_thumbnail_data)
     g_free(gimp_thumbnail_data);
-  gint x = gimp_image_width(image_ID);
-  gint y = gimp_image_height(image_ID);
+  gint x = gimp_image_width(p2gint(image_ID));
+  gint y = gimp_image_height(p2gint(image_ID));
+
+  /* GIMP limits thumbnails to 1024px! */
+  if (*width > 1024)
+    *width = 1024;
+  if (*height > 1024)
+    *height = 1024;
+
   if (*width > x)
     *width = x;
   if (*height > y)
@@ -477,7 +484,7 @@ run (const char        *name,		/* I - Name of print program. */
               gimp_vars.orientation = atoi(value);
               break;
             }
-
+            /* Intentional fallthrough */
           default:
             values[0].data.d_status = GIMP_PDB_CALLING_ERROR;
             fprintf(stderr,"Parameter unsupported in gimp2 plugin for parameter %s\n", key);
