@@ -149,6 +149,7 @@ print_debug_params(stp_vars_t *v)
   print_remote_int_param(v, "Bits", pd->bitwidth);
   print_remote_int_param(v, "Drop Size", pd->drop_size);
   print_remote_int_param(v, "Initial_vertical_offset", pd->initial_vertical_offset);
+  print_remote_int_param(v, "Printing_initial_vertical_offset", pd->printing_initial_vertical_offset);
   print_remote_int_param(v, "Channels_in_use", pd->channels_in_use);
   print_remote_int_param(v, "Logical_channels", pd->logical_channels);
   print_remote_int_param(v, "Physical_channels", pd->physical_channels);
@@ -250,6 +251,9 @@ escp2_set_remote_sequence(stp_vars_t *v)
       if (stp_check_float_parameter(pv, "PageDryTime", STP_PARAMETER_ACTIVE))
 	stp_send_command(v, "DR", "bcch", 0, 1,
 			 (int) stp_get_float_parameter(pv, "PageDryTime"));
+      if (stp_check_int_parameter(pv, "escp2_roll_lb", STP_PARAMETER_ACTIVE))
+	stp_send_command(v, "LB", "bccc", 0, 1,
+			 stp_get_int_parameter(pv, "escp2_roll_lb"));
       /* Next comes paper path */
       if (pd->input_slot)
 	{
@@ -405,12 +409,29 @@ escp2_set_dot_size(stp_vars_t *v)
     stp_send_command(v, "\033(e", "bcc", 0, pd->drop_size);
 }
 
+static void
+escp2_set_cutter(stp_vars_t *v)
+{
+  escp2_privdata_t *pd = get_privdata(v);
+
+  if (pd->roll_only)
+  {
+    int l = pd->page_true_height * 2880 / 72;
+    /* always 2880, independent of configured resolutions */
+
+    stp_send_command(v, "\033(g", "bl", l);
+  }
+}
+
 static void
 escp2_set_page_height(stp_vars_t *v)
 {
   escp2_privdata_t *pd = get_privdata(v);
   int l = (pd->page_true_height + pd->paper_extra_bottom) *
     pd->page_management_units / 72;
+
+  if (pd->roll_only) return;
+
   if (pd->use_extended_commands)
     stp_send_command(v, "\033(C", "bl", l);
   else
@@ -424,6 +445,8 @@ escp2_set_margins(stp_vars_t *v)
   int bot = pd->page_management_units * pd->page_bottom / 72;
   int top = pd->page_management_units * pd->page_top / 72;
 
+  if (pd->roll_only) return;
+
   top += pd->initial_vertical_offset;
   top -= pd->page_extra_height;
   bot += pd->page_extra_height;
@@ -502,6 +525,8 @@ static void
 set_color(stp_vars_t *v, stp_pass_t *pass, int color)
 {
   escp2_privdata_t *pd = get_privdata(v);
+  (void)pass;
+
   if (pd->last_color != color && ! pd->use_extended_commands)
     {
       int ncolor = pd->channels[color]->color;
@@ -521,6 +546,7 @@ set_horizontal_position(stp_vars_t *v, stp_pass_t *pass, int vertical_subpass)
   int microoffset = (vertical_subpass & (pd->horizontal_passes - 1)) *
     pd->image_scaled_width / pd->image_printed_width;
   int pos = pd->image_left_position + microoffset;
+  (void)pass;
 
   if (pos != 0)
     {
@@ -537,6 +563,8 @@ static void
 send_print_command(stp_vars_t *v, stp_pass_t *pass, int ncolor, int nlines)
 {
   escp2_privdata_t *pd = get_privdata(v);
+  (void)pass;
+
   int lwidth = (pd->image_printed_width + (pd->horizontal_passes - 1)) /
     pd->horizontal_passes;
   if (pd->command_set == MODEL_COMMAND_PRO || pd->variable_dots)
@@ -623,6 +651,7 @@ stpi_escp2_init_printer(stp_vars_t *v)
   escp2_set_printer_weave(v);
   escp2_set_printhead_speed(v);
   escp2_set_dot_size(v);
+  escp2_set_cutter(v);
   escp2_set_printhead_resolution(v);
   escp2_set_page_height(v);
   escp2_set_margins(v);
