@@ -1,6 +1,4 @@
 /*
- * "$Id: mxml-file.c,v 1.13 2012/01/19 13:26:48 m0m Exp $"
- *
  * File loading code for mini-XML, a small XML-like file parsing library.
  *
  * Copyright 2003 by Michael Sweet.
@@ -37,8 +35,11 @@
  * Include necessary headers...
  */
 
+#include <gutenprint/util.h>
 #include <gutenprint/mxml.h>
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
 #define MXML_BUFSIZE (64)
 #define ENTITY_BUFSIZE (64)
 
@@ -63,7 +64,7 @@ static int		mxml_write_node(stp_mxml_node_t *node, void *p,
 					int (*putc_cb)(int, void *));
 static int		mxml_write_string(const char *s, void *p,
 					  int (*putc_cb)(int, void *));
-static int		mxml_write_ws(stp_mxml_node_t *node, void *p, 
+static int		mxml_write_ws(stp_mxml_node_t *node, void *p,
 			              int (*cb)(stp_mxml_node_t *, int), int ws,
 				      int col, int (*putc_cb)(int, void *));
 
@@ -172,7 +173,7 @@ stp_mxmlSaveAllocString(stp_mxml_node_t *node,	/* I - Node to write */
     * return...
     */
 
-    return (strdup(buffer));
+    return (stp_strdup(buffer));
   }
 
  /*
@@ -431,6 +432,10 @@ mxml_load_data(stp_mxml_node_t *top,	/* I - Top node */
             node = stp_mxmlNewReal(parent, strtod(buffer, &bufptr));
 	    break;
 
+	case STP_MXML_DIMENSION :
+            node = stp_mxmlNewDimension(parent, strtod(buffer, &bufptr));
+	    break;
+
 	case STP_MXML_TEXT :
             node = stp_mxmlNewText(parent, whitespace, buffer);
 	    break;
@@ -438,7 +443,7 @@ mxml_load_data(stp_mxml_node_t *top,	/* I - Top node */
         default : /* Should never happen... */
 	    node = NULL;
 	    break;
-      }	  
+      }
 
       if (*bufptr)
       {
@@ -1141,7 +1146,7 @@ mxml_write_node(stp_mxml_node_t *node,	/* I - Node to write */
 	      return (-1);
             if ((*putc_cb)('\"', p) < 0)
 	      return (-1);
-	    
+
             col += strlen(attr->name) + strlen(attr->value) + 3;
 	  }
 
@@ -1244,6 +1249,7 @@ mxml_write_node(stp_mxml_node_t *node,	/* I - Node to write */
           break;
 
       case STP_MXML_REAL :
+      case STP_MXML_DIMENSION :
 	  if (node->prev)
 	  {
 	    if (col > STP_MXML_WRAP)
@@ -1460,8 +1466,3 @@ mxml_write_ws(stp_mxml_node_t *node,	/* I - Current node */
 
   return (col);
 }
-
-
-/*
- * End of "$Id: mxml-file.c,v 1.13 2012/01/19 13:26:48 m0m Exp $".
- */
