@@ -26,7 +26,9 @@
  */
 
 #include <gutenprint/mxml.h>
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
 
 
 /*
