@@ -90,11 +90,11 @@ typedef struct
 {
   const char *name;
   const char *text;
-  short hres;
-  short vres;
-  short printed_hres;
-  short printed_vres;
-  short vertical_passes;
+  unsigned short hres;
+  unsigned short vres;
+  unsigned short printed_hres;
+  unsigned short printed_vres;
+  unsigned short vertical_passes;
   stp_raw_t *command;
   stp_vars_t *v;
 } res_t;
@@ -110,10 +110,10 @@ typedef struct
 {
   char *name;
   char *text;
-  short min_hres;
-  short min_vres;
-  short max_hres;
-  short max_vres;
+  unsigned short min_hres;
+  unsigned short min_vres;
+  unsigned short max_hres;
+  unsigned short max_vres;
   short desired_hres;
   short desired_vres;
 } quality_t;
@@ -166,7 +166,7 @@ typedef struct
 typedef struct
 {
   const char *name;
-  short n_subchannels;
+  unsigned short n_subchannels;
   physical_subchannel_t *subchannels;
   const char *hue_curve_name;
   stp_curve_t *hue_curve;
@@ -188,8 +188,8 @@ typedef struct
 {
   const char *name;
   const char *text;
-  short channel_count;
-  short aux_channel_count;
+  unsigned short channel_count;
+  unsigned short aux_channel_count;
   inkset_id_t inkset;
   const stp_raw_t *init_sequence;
   const stp_raw_t *deinit_sequence;
@@ -199,7 +199,7 @@ typedef struct
 
 typedef struct
 {
-  int n_shades;
+  unsigned int n_shades;
   double *shades;
 } shade_t;
 
@@ -207,8 +207,8 @@ typedef struct
 {
   const char *name;
   const char *text;
-  short n_shades;
-  short n_inks;
+  unsigned short n_shades;
+  unsigned short n_inks;
   const stp_raw_t *init_sequence;
   const stp_raw_t *deinit_sequence;
   shade_t *shades;
@@ -218,7 +218,7 @@ typedef struct
 typedef struct
 {
   const char *name;
-  short n_inklists;
+  unsigned short n_inklists;
   inklist_t *inklists;
 } inkgroup_t;
 
@@ -304,6 +304,10 @@ typedef struct
 #define MODEL_ENVELOPE_LANDSCAPE_NO	0x0000ul
 #define MODEL_ENVELOPE_LANDSCAPE_YES	0x4000ul
 
+#define MODEL_ROLL_ONLY_MASK		0x8000ul /* Only supports roll-feed */
+#define MODEL_ROLL_ONLY_NO		0x0000ul
+#define MODEL_ROLL_ONLY_YES		0x8000ul
+
 typedef enum
 {
   MODEL_COMMAND,
@@ -316,6 +320,7 @@ typedef enum
   MODEL_PACKET_MODE,
   MODEL_INTERCHANGEABLE_INK,
   MODEL_ENVELOPE_LANDSCAPE,
+  MODEL_ROLL_ONLY,
   MODEL_LIMIT
 } escp2_model_option_t;
 
@@ -347,10 +352,10 @@ typedef struct escp2_printer
 				      /* must use color parameters */
 				      /* rather than (faster) black */
 				      /* only parameters*/
-  short		max_hres;
-  short		max_vres;
-  short		min_hres;
-  short		min_vres;
+  unsigned short	max_hres;
+  unsigned short	max_vres;
+  unsigned short	min_hres;
+  unsigned short	min_vres;
 /*****************************************************************************/
   /* Miscellaneous printer-specific data */
   short		extra_feed;	/* Extra distance the paper can be spaced */
@@ -367,6 +372,7 @@ typedef struct escp2_printer
   short		micro_left_margin; /* Precise left margin (base separation) */
   short		initial_vertical_offset;
   short		black_initial_vertical_offset;
+  short		printing_initial_vertical_offset;
   short		extra_720dpi_separation;
   short		min_horizontal_position_alignment; /* Horizontal alignment */
 					       /* for good performance */
@@ -543,6 +549,7 @@ typedef struct
   model_featureset_t command_set; /* Which command set this printer supports */
   int variable_dots;		/* Print supports variable dot sizes */
   int has_graymode;		/* Printer supports fast grayscale mode */
+  int roll_only;		/* Printer only supports roll-fed media */
   int base_separation;		/* Basic unit of separation */
   int resolution_scale;		/* Scale factor for ESC(D command */
   int separation_rows;		/* Row separation scaling */
