@@ -21,11 +21,13 @@
  * 02110-1301 USA
  */
 
+#include <ctype.h>
 #include <stdio.h>
 #include <gutenprint/mxml.h>
 #include <gutenprint/util.h>
 #include <gutenprint/string-list.h>
 #include <stdlib.h>
+#include <strings.h>
 #include "xmlppd.h"
 
 typedef struct
@@ -210,8 +212,8 @@ stpi_xmlppd_read_ppd_file(const char *filename)	/* I - PPD file */
 		*text,			/* Pointer to text */
 		*value;			/* Pointer to value */
   order_t	*order_array;		/* Precedence order of options */
-  int		i;
-  int		option_count;
+  unsigned int	i;
+  unsigned int	option_count;
   int		order_length;
   char		*order_list;
   stp_string_list_t *ialist = stp_string_list_create();
@@ -495,7 +497,7 @@ stpi_xmlppd_read_ppd_file(const char *filename)	/* I - PPD file */
       option = NULL;
       stp_option_data_name[0] = '\0';
     }
-      
+
   for (i = 0; i < stp_string_list_count(ialist); i++)
     {
       stp_param_string_t *pstr = stp_string_list_param(ialist, i);
