@@ -1,5 +1,4 @@
 /*
- * "$Id: testpattern.h,v 1.20 2010/09/06 17:08:39 rlk Exp $"
  *
  *   Test pattern generator for Gimp-Print
  *
@@ -16,8 +15,7 @@
  *   for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -37,10 +35,10 @@ typedef struct
   } type;
   union {
     struct {
-      double mins[32];
-      double vals[32];
-      double gammas[32];
-      double levels[32];
+      double mins[STP_CHANNEL_LIMIT];
+      double vals[STP_CHANNEL_LIMIT];
+      double gammas[STP_CHANNEL_LIMIT];
+      double levels[STP_CHANNEL_LIMIT];
       double lower;
       double upper;
     } pattern;
@@ -83,6 +81,7 @@ extern size_mode_t global_size_mode;
 extern double global_hsize;
 extern double global_vsize;
 extern int global_noblackline;
+extern int global_colorline;
 extern const char *global_image_type;
 extern int global_color_model;
 extern int global_bit_depth;
@@ -90,6 +89,7 @@ extern int global_channel_depth;
 extern int global_did_something;
 extern int global_invert_data;
 extern int global_noscale;
+extern int global_round_size;
 extern char *global_output;
 extern int global_quiet;
 extern FILE *output;
@@ -109,5 +109,3 @@ typedef struct yylv {
 #define YYSTYPE_IS_DECLARED 1
 
 #include "testpatterny.h"
-
-
