@@ -1,4 +1,3 @@
-/* $Id: unprint.c,v 1.44 2009/03/07 19:38:00 rlk Exp $ */
 /*
  * Generate PPM files from printer output
  *
@@ -17,8 +16,7 @@
  *   for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -51,17 +49,17 @@ typedef struct {
   int page_management_units; /* dpi */
   int relative_horizontal_units; /* dpi */
   int absolute_horizontal_units; /* dpi, assumed to be >= relative */
-  int relative_vertical_units; /* dpi */
-  int absolute_vertical_units; /* dpi, assumed to be >= relative */
+  unsigned int relative_vertical_units; /* dpi */
+  unsigned int absolute_vertical_units; /* dpi, assumed to be >= relative */
   int horizontal_spacing;	/* Horizontal dot spacing */
-  int top_margin; /* dots */
-  int bottom_margin; /* dots */
-  int page_height; /* dots */
+  unsigned int top_margin; /* dots */
+  unsigned int bottom_margin; /* dots */
+  unsigned int page_height; /* dots */
   int dotsize;
   int bpp; /* bits per pixel */
   int current_color;
   int xposition; /* dots */
-  int yposition; /* dots */
+  unsigned int yposition; /* dots */
   int monomode;
   int nozzle_separation;
   int nozzles;
@@ -69,8 +67,8 @@ typedef struct {
   int got_graphics;
   int left_edge;
   int right_edge;
-  int top_edge;
-  int bottom_edge;
+  unsigned int top_edge;
+  unsigned int bottom_edge;
   quadtone_t quadtone;
 } pstate_t;
 
@@ -192,8 +190,8 @@ extern void merge_line(line_type *p, unsigned char *l, int startl, int stopl,
 extern void expand_line (unsigned char *src, unsigned char *dst, int height,
 			 int skip, int left_ignore);
 extern void write_output (FILE *fp_w, int dontwrite, int allblack);
-extern void find_white (unsigned char *buff,int npix, int *left, int *right);
-extern int update_page (unsigned char *buff, int buffsize, int m, int n,
+extern void find_white (unsigned char *buff,unsigned int npix, unsigned int *left, unsigned int *right);
+extern int update_page (unsigned char *buff, unsigned int buffsize, unsigned int m, unsigned int n,
 			int color, int density);
 extern void parse_escp2 (FILE *fp_r);
 extern void reverse_bit_order (unsigned char *buff, int n);
@@ -418,7 +416,7 @@ expand_line (unsigned char *src, unsigned char *dst, int height, int skip,
   /*
    * src is a pointer to a bit stream which is composed of fields of height
    * bpp starting with the most significant bit of the first byte and
-   * proceding from there with no regard to byte boundaries.  For the
+   * proceeding from there with no regard to byte boundaries.  For the
    * existing Epson printers, bpp is 1 or 2, which means fields will never
    * cross byte boundaries.  However, if bpp were 3, this would undoubtedly
    * happen.  This routine will make no assumptions about bpp, and handle each
@@ -467,8 +465,14 @@ write_output(FILE *fp_w, int dontwrite, int allblack)
   right = pstate.right_edge;
   height = oversample * (last - first + 1);
 
-  fprintf(stderr, "Image from (%d,%d) to (%d,%d) (%dx%d) (%.3fx%.3f)\n",
-	  left, first, right, last, right - left + 1, last - first + 1,
+  fprintf(stderr, "Image from (%d,%d) (%.3fx%.3f) to (%d,%d) (%.3fx%.3f) size (%dx%d) (%.3fx%.3f)\n",
+	  left, first - pstate.top_margin,
+	  (left / (double) pstate.page_management_units),
+	  (first / (double) pstate.page_management_units),
+	  right, last - pstate.top_margin,
+	  (right / (double) pstate.page_management_units),
+	  (last / (double) pstate.page_management_units),
+	  right - left + 1, last - first + 1,
 	  (right - left + 1) / (double) pstate.page_management_units,
 	  (last - first + 1) / (double) pstate.page_management_units);
 
@@ -479,6 +483,8 @@ write_output(FILE *fp_w, int dontwrite, int allblack)
   out_row = stp_malloc(sizeof(ppmpixel) * width);
   fprintf(stderr, "Writing output...\n");
 
+  if (dontwrite)
+    return;
   /* write out the PPM header */
   fprintf(fp_w, "P6\n");
   fprintf(fp_w, "%d %d\n", width, height);
@@ -495,30 +501,23 @@ write_output(FILE *fp_w, int dontwrite, int allblack)
 	      float *ink = ink_colors[inknum];
 	      if (lt->line[c])
 		{
-		  if (dontwrite)
-		    donothing += lt->line[c][0] ^
-		      lt->line[c][lt->stopx[c] - lt->startx[c] + 1];
-		  else
+		  for (p = lt->startx[c]; p <= lt->stopx[c]; p++)
 		    {
-		      for (p = lt->startx[c]; p <= lt->stopx[c]; p++)
-			{
-			  amount = get_bits(lt->line[c], p - lt->startx[c]);
-			  mix_ink(out_row[p - left], c, amount, ink,
-				  pstate.quadtone);
-			}
+		      amount = get_bits(lt->line[c], p - lt->startx[c]);
+		      mix_ink(out_row[p - left], c, amount, ink,
+			      pstate.quadtone);
 		    }
 		}
 	    }
 	}
-      if (!dontwrite)
-	for (i = 0; i < oversample; i++)
-	  fwrite(out_row, sizeof(ppmpixel), width, fp_w);
+      for (i = 0; i < oversample; i++)
+	fwrite(out_row, sizeof(ppmpixel), width, fp_w);
     }
   stp_free(out_row);
 }
 
 void
-find_white(unsigned char *buff,int npix, int *left, int *right)
+find_white(unsigned char *buff,unsigned int npix, unsigned int *left, unsigned int *right)
 {
 
   /*
@@ -526,8 +525,8 @@ find_white(unsigned char *buff,int npix, int *left, int *right)
    * pixels and fill that info into left and right.
    */
 
-  int i, j, max;
-  int words, bytes, bits, extra;
+  unsigned int i, j, max;
+  unsigned int words, bytes, bits, extra;
 
   *left = *right = 0;
   bits = npix * pstate.bpp;
@@ -607,18 +606,21 @@ find_white(unsigned char *buff,int npix, int *left, int *right)
 
 int
 update_page(unsigned char *buff, /* I - pixel data               */
-	    int buffsize,        /* I - size of buff in bytes     */
-	    int m,              /* I - height of area in pixels */
-	    int n,              /* I - width of area in pixels  */
-	    int color,          /* I - color of pixel data      */
-	    int density         /* I - horizontal density in dpi  */
+	    unsigned int buffsize, /* I - size of buff in bytes     */
+	    unsigned int m,      /* I - height of area in pixels */
+	    unsigned int n,      /* I - width of area in pixels  */
+	    int color,           /* I - color of pixel data      */
+	    int density          /* I - horizontal density in dpi  */
 	    )
 {
-  int y, skip, oldstart, oldstop, mi = 0;
-  int left_white, right_white, width;
+  unsigned int y;
+  int skip, oldstart, oldstop, mi = 0;
+  unsigned int left_white, right_white, width;
   unsigned char *oldline;
   int sep;
 
+  (void)buffsize;
+
   if ((n == 0) || (m == 0))
     return(0);  /* shouldn't happen */
 
@@ -768,10 +770,10 @@ do									\
 static void
 parse_escp2_data(FILE *fp_r)
 {
-  int i, m = 0, n = 0, c = 0;
+  unsigned int i, m = 0, n = 0, c = 0;
   int currentcolor = 0;
   int density = 0;
-  int bandsize;
+  unsigned int bandsize;
   switch (ch)
     {
     case 'i':
@@ -1383,7 +1385,7 @@ parse_canon(FILE *fp_r)
 
   int m=0;
   int currentcolor,currentbpp,density,l_eject;
-  int cmdcounter;
+//  int cmdcounter;
   int delay_c=0, delay_m=0, delay_y=0, delay_C=0,
     delay_M=0, delay_Y=0, delay_K=0, currentdelay=0;
 
@@ -1429,7 +1431,7 @@ parse_canon(FILE *fp_r)
    switch (ch) {
    case '[': /* 0x5b initialize printer */
      get1("Error reading CEM-code.\n");
-     cmdcounter= global_counter;
+//     cmdcounter= global_counter;
      get2("Error reading CEM-data size.\n");
      getn(sh,"Error reading CEM-data.\n");
 
@@ -1476,7 +1478,7 @@ parse_canon(FILE *fp_r)
 
    case '(': /* 0x28 */
      get1("Corrupt file.  Incomplete extended command.\n");
-     cmdcounter= global_counter;
+//     cmdcounter= global_counter;
      get2("Corrupt file.  Error reading buffer size.\n");
      bufsize=sh;
      getn(bufsize,"Corrupt file.  Error reading data buffer.\n");
@@ -1612,6 +1614,17 @@ main(int argc,char *argv[])
 	      else
 		fp_r = stdin;
 	      break;
+	    case 'h':
+	      fprintf(stderr, "Usage: %s [-m mask] [-n nozzle_sep] [-s extra] [-b] [-q] [-Q] [-M] [-u] [in [out]]\n", argv[0]);
+	      fprintf(stderr, "        -m mask       Color mask to unprint\n");
+	      fprintf(stderr, "        -n nozzle_sep Nozzle separation in vertical units for old printers\n");
+	      fprintf(stderr, "        -s extra      Extra feed requirement for old printers (typically 1)\n");
+	      fprintf(stderr, "        -b            Unprint everything in black\n");
+	      fprintf(stderr, "        -q            Don't produce output\n");
+	      fprintf(stderr, "        -Q            Assume quadtone inks\n");
+	      fprintf(stderr, "        -M            Assume MIS quadtone inks\n");
+	      fprintf(stderr, "        -u            Unweave\n");
+	      return 1;
 	    case 'm':
 	      if (argv[arg][2])
 		{
@@ -1702,15 +1715,15 @@ main(int argc,char *argv[])
 	{
 	  if (fp_r)
 	    {
-	      if (!(fp_w = fopen(argv[arg],"w")))
+	      if (!(fp_w = fopen(argv[arg],"wb")))
 		{
-		  perror("Error opening ouput file");
+		  perror("Error opening output file");
 		  exit(-1);
 		}
 	    }
 	  else
 	    {
-	      if (!(fp_r = fopen(argv[arg],"r")))
+	      if (!(fp_r = fopen(argv[arg],"rb")))
 		{
 		  perror("Error opening input file");
 		  exit(-1);
